/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.bloom;
import ostore.util.Carp;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.Pair;
import java.util.Map;
import java.util.HashMap;

public class SimulationLog {

    public static class ReadTransactionInfo {
	
	public NodeId source;
	public SecureHash source_guid;
	public SecureHash guid;
	public long location_start_time;
	public boolean started_location_as_bloom;
	public boolean finished_location_as_bloom;
	public long location_finish_time;
	public long location_distance;
	public long ideal_location_distance;

	public NodeId provider;
	public long routing_start_time;
	public boolean started_routing_as_bloom;
	public boolean finished_routing_as_bloom;
	public long routing_finish_time;
	public long routing_ideal_lat;

	public long total_ideal_lat;

	public int location_distance_hops;
	public int bloom_obj_hops;
	public int bloom_obj_net_hops;
	public int tap_obj_hops;
	public int tap_obj_net_hops;
	public int tap_srv_hops;
	public int tap_srv_net_hops;
	public int hn_net_hops;
	public int hn_c_net_hops;
	public int dead_ends;

	public String toString () {
	    return location_start_time + " " + 
		source.port () + " " + guid + " " + 
		(location_finish_time - location_start_time) + " " +
		ideal_location_distance + " " +
		(routing_finish_time - location_start_time) + " " +
		total_ideal_lat + " " + 
		(finished_location_as_bloom ? "B" : 
		(started_location_as_bloom ? "D" : "T")) +
		" " + bloom_obj_hops + " " + provider.port () + " " +
		location_distance;
	}
    }

    private static final void check (boolean expr) {
	if(!expr) Carp.die("BUG");
    }

    public SimulationLog () {
	_by_node_id = new HashMap ();
	_by_node_guid = new HashMap ();
    }

    public ReadTransactionInfo new_xact (
	    NodeId src, SecureHash src_guid, SecureHash g) {
	ReadTransactionInfo result = new ReadTransactionInfo ();
	result.source = src;
	result.source_guid = src_guid;
	result.guid = g;
	Pair p1 = new Pair (src, g);
	Pair p2 = new Pair (src_guid, g);
	check (! _by_node_id.containsKey (p1));
	check (! _by_node_guid.containsKey (p2));
	_by_node_id.put (p1, result);
	_by_node_guid.put (p2, result);
	return result;
    }

    public ReadTransactionInfo xact (NodeId s, SecureHash g) {
	return (ReadTransactionInfo) _by_node_id.get (new Pair (s, g));
    }

    public ReadTransactionInfo xact (SecureHash s, SecureHash g) {
	return (ReadTransactionInfo) _by_node_guid.get (new Pair (s, g));
    }

    public void xact_finished (NodeId s, SecureHash g) {
	ReadTransactionInfo xact = 
	    (ReadTransactionInfo) _by_node_id.remove (new Pair (s, g));
	check (xact != null);
	Object foo = _by_node_guid.remove (new Pair (xact.source_guid, g));
	check (foo != null);
	System.out.println (xact);
    }

    public static SimulationLog instance (NodeId node_id) {
	return _instance;
    }

    private static SimulationLog _instance;
    static {
	_instance = new SimulationLog ();
    }

    private Map _by_node_id;
    private Map _by_node_guid;
}

