/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.bloom;
import ostore.util.ByteUtils;
import ostore.util.NodeId;
import ostore.network.NetworkMessage;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;

/**
 * Used to exert backpressure on update traffic.
 *
 * @author     Sean C. Rhea
 * @version    $Id: TopologyDiscoverResponse.java,v 1.12 2002/07/20 19:38:01 srhea Exp $
 */
public class TopologyDiscoverResponse 
extends NetworkMessage implements QuickSerializable {

    public NodeId update_source;
    public boolean on_shortest_path;

    public TopologyDiscoverResponse (NodeId peer, NodeId update_source,
	    boolean on_shortest_path) {
	super (peer, false /* outbound */);
	this.update_source = update_source;
	this.on_shortest_path = on_shortest_path;
    }

    public TopologyDiscoverResponse (InputBuffer buffer) throws QSException {
	super (buffer);
	update_source = (NodeId) buffer.nextObject ();
	on_shortest_path = buffer.nextBoolean ();
    }



    public void serialize (OutputBuffer buffer) {
	super.serialize (buffer);
	buffer.add (update_source);
	buffer.add (on_shortest_path);
    }

    public Object clone () throws CloneNotSupportedException {
	TopologyDiscoverResponse result = 
	    (TopologyDiscoverResponse) super.clone ();
	result.update_source = update_source;
	result.on_shortest_path = on_shortest_path;
	return result;
    }

    public String toString () {
	return "(TopologyDiscoverResponse " + (inbound ? "from " : "to ") +
	    peer.port () + " concerning source " + update_source.port () + 
	    " " + on_shortest_path + ")";
    }
}

