/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.monitor;

import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.SinkIF;
import ostore.util.*;
import java.util.*;
import java.io.*;

/**
 * 1-minute CPU Load average. This keeps track of one value:
 * <pre>
 * [0] = 1-minute load average
 * </pre>
 *
 * @author  Jeff Pang
 * @version $Id: LoadAvgResource.java,v 1.2 2003/02/18 05:33:20 jeffpang Exp $
 **/
public class LoadAvgResource extends DynamicResource
{

    private RandomAccessFile loadavg;

    public LoadAvgResource(NodeId id, int sampleInterval) throws IOException
    {
	super(id, "loadavg", 0, sampleInterval); // beter way to do this?
	// TODO: how to do this on non-linux
	loadavg = new RandomAccessFile("/proc/loadavg", "r");
	sample();
    }
    
    /**
     * Take a sample at the current time.
     */
    public void sample()
    {
	byte[] str = new byte[4];
	double val = 0;
	try {
	    loadavg.seek(0);
	    loadavg.read(str);
	    val = Double.parseDouble(new String(str));
	} catch (Exception ex) {
	    Carp.die("LoadAvgResource.sample: exception: " + ex);
	}
	log(val);
    }

    
    public static void main(String[] args) throws Exception
    {
	// Test
	LoadAvgResource rsc = new LoadAvgResource(null, 1000);

	while (true) {
	    Thread.currentThread().sleep(rsc.getSampleInterval());
	    rsc.sample();
	    System.out.println(rsc.getLog().util(0));
	}
    }

}
