/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.monitor;

import seda.sandStorm.api.*;
import ostore.util.*;

/**
 * Response to a <tt>RequestResourceReq</tt>.
 *
 * @author  Jeff Pang
 * @version $Id: RequestResourceResp.java,v 1.2 2003/02/21 20:37:16 jeffpang Exp $
 **/
public class RequestResourceResp implements QueueElementIF
{
    /**
     * Successful request.
     */
    public static final int SUCCESS = 1;
    /**
     * No resource with the name in request.
     */
    public static final int NO_SUCH_RESOURCE = 2;

    private String name;
    private Resource rsrc;
    private int code;
    
    /**
     * Successful request.
     *
     * @param req the original request.
     * @param rsrc the handle to the resource requested.
     */
    public RequestResourceResp(RequestResourceReq req, Resource rsrc)
    {
	this.name = req.getName();
	this.rsrc = rsrc;
	this.code = SUCCESS;
    }

    /**
     * Unsuccessful request.
     *
     * @param req the original request.
     * @param code error code.
     */
    public RequestResourceResp(RequestResourceReq req, int code)
    {
	this.name = req.getName();
	this.code = code;
    }

    /**
     * Get the name of the resource handle requested.
     */
    public String getName()
    {
	return name;
    }

    /**
     * Get the handle to the resource.
     */
    public Resource getResource()
    {
	return rsrc;
    }

    /**
     * Get status code.
     */
    public int getCode()
    {
	return code;
    }

    public boolean success()
    {
	return code == SUCCESS;
    }
}
