/**
 * QSRsaPublicKey.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSRsaPublicKey.java,v 1.13 2002/08/02 19:39:56 geels Exp $
 */

package ostore.security;
import cryptix.jce.provider.rsa.RSAPublicKeyCryptix;
import java.math.BigInteger;
import java.security.PublicKey;
import ostore.util.ByteUtils;
import ostore.util.Carp;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.SHA1Hash;

/**
 * A subtype of <code>QSPublicKey</code> for RSA public keys.
 * 
 * @author   Sean C. Rhea
 * @version  $Id: QSRsaPublicKey.java,v 1.13 2002/08/02 19:39:56 geels Exp $
 */
public class QSRsaPublicKey implements QSPublicKey {

    private static final String class_tag = "QSRsaPublicKey";

    public String toString () {
	return "(QSRsaPublicKey " + (new SHA1Hash (this)) + ")";
    }

    public QSRsaPublicKey (PublicKey key) {
	final String tag = class_tag + ".QSRsaPublicKey";
	if (key == null ) Carp.die( tag + ": key must be non-null.");
	if (!(key instanceof RSAPublicKeyCryptix)) 
	    Carp.die( tag + ": key must be of type RSAPublicKeyCryptix, " +
		"not of type " + key.getClass ().getName () + ".");
	_key = (RSAPublicKeyCryptix) key;
    }

    public QSRsaPublicKey (InputBuffer buffer) throws QSException {
	BigInteger e = buffer.nextBigInteger ();
	BigInteger n = buffer.nextBigInteger ();
	_key = new RSAPublicKeyCryptix (n, e);
    }

    public void serialize (OutputBuffer buffer ) {
	buffer.add (_key.getPublicExponent ());
	buffer.add (_key.getModulus ());
    }

    public PublicKey key () {
	return _key;
    }

    public boolean equals (Object other) {
	if (other == null)
	    return false;
	if (other == this)
	    return true;
	if (! (other instanceof QSRsaPublicKey))
	    return false;
	QSRsaPublicKey rhs = (QSRsaPublicKey) other;
	return _key.getModulus  ().equals (rhs._key.getModulus  ()) &&
	       _key.getPublicExponent ().equals (rhs._key.getPublicExponent ());
    }

    protected RSAPublicKeyCryptix _key;
}

