/**
 * QSSignature.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSSignature.java,v 1.8 2002/07/20 19:18:15 srhea Exp $
 */

package ostore.security;
import ostore.util.*;
import java.security.*;

/**
 * A class used to map a particular digital signature algorithm and
 * provider to a type code, for more efficient encoding.  
 * 
 * @author   Sean C. Rhea
 * @version  $Id: QSSignature.java,v 1.8 2002/07/20 19:18:15 srhea Exp $
 *
 * @see SignedQS
 */
public interface QSSignature extends QuickSerializable {

    /**
     * The signing algorithm.  Using this and the value of
     * {@link #provider}, a user of this class should be able to produce a
     * signature engine to pass to {@link SignedQS#verify} by calling {@link
     * java.security.Signature#getInstance(String, String)}.
     *
     * @return the name of the digital signature algorithm, for instance, 
     *         "SHA-1/RSA/PKCS#1"
     */
    public String algorithm ();

    /**
     * The signing algorithm's provider, as used with
     * <code>getInstance</code>.  Using this and the value of {@link
     * #algorithm}, a user of this class should be able to produce a
     * signature engine to pass to {@link SignedQS#verify} by calling
     * {@link java.security.Signature#getInstance(String, String)}.
     *
     * @return the name of the digital signature algorithm provider, for
     * instance, "CryptixCrypto"
     */
    public String provider ();
}

