/**
 * gmp-interface.cc
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

#include <jni.h>
#include <malloc.h>
#include <stdio.h>
#include <gmp.h>
#include <iostream>
#include <openssl/sha.h>
#include <openssl/evp.h>
#include <assert.h>
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>

#define PROFILE false

/* mpz_out_raw -- Output a mpz_t in binary.  Use an endianess and word size
   independent format.

Copyright 1995, 2001 Free Software Foundation, Inc.

This file is part of the GNU MP Library.

The GNU MP Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.

The GNU MP Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public License
along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. */

#include <stdio.h>

#include "gmp.h"
//#include "gmp-impl.h"

#define BITS_PER_CHAR  8
static const int BYTES_PER_MP_LIMB = sizeof(int);

static void fputc (unsigned char x, unsigned char *out_bytes) {
    *out_bytes = x;
}

size_t
mpz_out_raw (unsigned char **out_bytes, unsigned int *out_len, mpz_srcptr x)
{
  int i;
  mp_size_t s;
  mp_size_t xsize = abs (x->_mp_size);
  mp_srcptr xp = x->_mp_d;
  mp_size_t out_bytesize;
  mp_limb_t hi_limb;
  int n_bytes_in_hi_limb;

  /*if (stream == 0)
    stream = stdout;*/

  if (xsize == 0)
    {
      /*for (i = 4 - 1; i >= 0; i--)
	fputc (0, stream);*/
	*out_bytes = (unsigned char*) malloc (1);
	**out_bytes = 0;
	*out_len = 1;
      return 1;
    }

  hi_limb = xp[xsize - 1];
  for (i = BYTES_PER_MP_LIMB - 1; i > 0; i--)
    {
      if ((hi_limb >> i * BITS_PER_CHAR) != 0)
	break;
    }
  n_bytes_in_hi_limb = i + 1;
  out_bytesize = BYTES_PER_MP_LIMB * (xsize - 1) + n_bytes_in_hi_limb;
  if (x->_mp_size < 0)
    out_bytesize = -out_bytesize;

  /* Make the size 4 bytes on all machines, to make the format portable.  */
  //for (i = 4 - 1; i >= 0; i--)
  //    fputc ((out_bytesize >> (i * BITS_PER_CHAR)) % (1 << BITS_PER_CHAR),
  //	   stream);
  *out_len = out_bytesize;
  *out_bytes = (unsigned char*) malloc (out_bytesize);

  /* Output from the most significant limb to the least significant limb,
     with each limb also output in decreasing significance order.  */

  /* Output the most significant limb separately, since we will only
     output some of its bytes.  */
     int offset = 0;
  for (i = n_bytes_in_hi_limb - 1; i >= 0; i--)
    fputc ((hi_limb >> (i * BITS_PER_CHAR)) % (1 << BITS_PER_CHAR), 
	    *out_bytes + offset++);

  /* Output the remaining limbs.  */
  for (s = xsize - 2; s >= 0; s--)
    {
      mp_limb_t x_limb;

      x_limb = xp[s];
      for (i = BYTES_PER_MP_LIMB - 1; i >= 0; i--)
	fputc ((x_limb >> (i * BITS_PER_CHAR)) % (1 << BITS_PER_CHAR),
	*out_bytes + offset++); }
  return /*ferror (stream) ? 0 :*/ abs (out_bytesize);
}

static bool _little_endian () {
    int i = 1;
    return ((unsigned char*) &i) [0] == 1;
}

static bool little_endian = _little_endian ();

void mpz2mag (mpz_t v, unsigned char **bytes, unsigned int *len) {
    mpz_out_raw (bytes, len, v); 
}

/* mpz_inp_raw -- Input a mpz_t in raw, but endianess, and wordsize
   independent format (as output by mpz_out_raw).

Copyright 1991, 1993, 1994, 1995, 2001 Free Software Foundation, Inc.

This file is part of the GNU MP Library.

The GNU MP Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.

The GNU MP Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public License
along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. */

#include <stdio.h>
#ifndef MPN_NORMALIZE
#define MPN_NORMALIZE(DST, NLIMBS) \
  do {                                                                  \
    while (NLIMBS > 0)                                                  \
      {                                                                 \
        if ((DST)[(NLIMBS) - 1] != 0)                                   \
          break;                                                        \
        NLIMBS--;                                                       \
      }                                                                 \
  } while (0)
#endif

#include "gmp.h"

#define BITS_PER_CHAR  8

size_t
mpz_inp_raw (mpz_ptr x, unsigned char *in_bytes, mp_size_t in_bytesize)
{
  int i;
  mp_size_t s;
  mp_size_t xsize;
  mp_ptr xp;
  unsigned int c;
  mp_limb_t x_limb;
  int neg_flag;

  /*if (stream == 0)
    stream = stdin;*/

  /* Read 4-byte size */
  /*in_bytesize = 0;
  for (i = 4 - 1; i >= 0; i--)
    {
      c = fgetc (stream);
      in_bytesize = (in_bytesize << BITS_PER_CHAR) | c;
    }
    */

  /* Size is stored as a 32 bit word; sign extend in_bytesize for non-32 bit
     machines.  */
  if (sizeof (mp_size_t) > 4)
    in_bytesize |= (-(in_bytesize < 0)) << 31;

  neg_flag = in_bytesize < 0;
  in_bytesize = abs (in_bytesize);
  xsize = (in_bytesize + BYTES_PER_MP_LIMB - 1) / BYTES_PER_MP_LIMB;

  if (xsize == 0)
    {
      x->_mp_size = 0;
      return 0;			/* we've read 4 bytes */
    }

  if (x->_mp_alloc < xsize)
    _mpz_realloc (x, xsize);
  xp = x->_mp_d;

    int offset = 0;
  x_limb = 0;
  for (i = (in_bytesize - 1) % BYTES_PER_MP_LIMB; i >= 0; i--)
    {
      c = *in_bytes++; //fgetc (stream);
      x_limb = (x_limb << BITS_PER_CHAR) | c;
    }
  xp[xsize - 1] = x_limb;

  for (s = xsize - 2; s >= 0; s--)
    {
      x_limb = 0;
      for (i = BYTES_PER_MP_LIMB - 1; i >= 0; i--)
	{
	  c = *in_bytes++; //fgetc (stream);
	  x_limb = (x_limb << BITS_PER_CHAR) | c;
	}
      xp[s] = x_limb;
    }

  /*if (c == EOF)
    return 0;	*/		/* error */

  MPN_NORMALIZE (xp, xsize);
  x->_mp_size = neg_flag ? -xsize : xsize;
  return in_bytesize;
}

void mag2mpz (mpz_t rop, unsigned char *bytes, unsigned int len) {
    mpz_inp_raw (rop, bytes, len);
}

void bigint2mpz (mpz_t rop,
	JNIEnv *env, jobject x, jmethodID big_int_to_byte_array) {
    jbyteArray ja = (jbyteArray) 
	env->CallObjectMethod (x, big_int_to_byte_array);
    unsigned char *bytes = (unsigned char*) env->GetByteArrayElements (ja, 0);
    jsize len = env->GetArrayLength (ja);
    mag2mpz (rop, bytes, len);
    env->ReleaseByteArrayElements (ja, (jbyte*) bytes, 0);
}

jobject mpz2bigint (
	JNIEnv *env, mpz_t x, jclass
	big_int_class, jmethodID big_int_init) {
    unsigned int len = 0;
    unsigned char *bytes = 0;
    mpz2mag (x, &bytes, &len);
    /*for (int i = 0; i < len; ++i) {
	printf ("%02x", (((int) bytes [i]) & 0xff));
    }
    printf ("\n");*/
    jbyteArray ja = env->NewByteArray (len+1);
    unsigned char* tmp = (unsigned char*) env->GetByteArrayElements (ja, 0);
    for (int i = 0; i < len; ++i) {
	tmp [i+1] = bytes [i];
    }
    free (bytes);
    tmp [0] = 0;
    env->ReleaseByteArrayElements (ja, (jbyte*) tmp, 0);
    return env->NewObject (big_int_class, big_int_init, ja);
}

void print_mpz (const char *name, mpz_t op) {
    fprintf (stdout, "%s = ", name);
    mpz_out_str (stdout, 16, op);
    fprintf (stdout, "\n");
}

void add_to_hash (EVP_MD_CTX *md, mpz_t value, jboolean debug) 
{
    unsigned char *buf = 0;
    unsigned int len = 0;

    mpz2mag (value, &buf, &len);
    if (debug) {
	for (int i = 0; i < len; ++i) {
	    printf ("%02x", (((int) buf [i]) & 0xff));
	}
	printf ("\n");
    }
	    
    EVP_DigestUpdate (md, buf, len);
    free (buf);
}

