/**
 * Verifier.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security.thresh;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import ostore.security.SafeBigInt;
import ostore.util.ByteUtils;
import ostore.util.QuickSerializable;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.TypeTable;

/**
 * Verifier for signature shares.
 *
 * <p>Reference: "Practical Threshold Signatures",
 * Victor Shoup (sho@zurich.ibm.com), IBM Research Paper RZ3121, 4/30/99
 *
 * @author  <a href=mailto:sweis@theory.lcs.mit.edu>Steve Weis</a> 
 * @version $Id: Verifier.java,v 1.15 2002/07/20 19:38:03 srhea Exp $
 */
public class Verifier implements QuickSerializable {

    private static final String class_tag = "Verifier";

    private SafeBigInt z;
    private SafeBigInt c;
    private SafeBigInt groupVerifier;
    private SafeBigInt shareVerifier;

    private int id;

    public Verifier(int id, SafeBigInt z, SafeBigInt c,
		    SafeBigInt shareVerifier,
		    SafeBigInt groupVerifier) {
	this.id = id;
	this.z = z;
	this.c = c;
	this.shareVerifier = shareVerifier;
	this.groupVerifier = groupVerifier;
    }

    public SafeBigInt getZ() {
	return this.z;
    }

    public SafeBigInt getShareVerifier() {
	return this.shareVerifier;
    }
    public SafeBigInt getGroupVerifier() {
	return this.groupVerifier;
    }

    public SafeBigInt getC() {
	return this.c;
    }

    /*
    public Verifier (byte [] data, int [] offset) throws QSException {
	id = ByteUtils.bytesToInt (data, offset);
	z = new SafeBigInt (data, offset);
	c = new SafeBigInt (data, offset);
	groupVerifier = new SafeBigInt (data, offset);
	shareVerifier = new SafeBigInt (data, offset);
    }
    */

    public Verifier (InputBuffer buffer) throws QSException {
	id = buffer.nextInt ();
	z = (SafeBigInt) buffer.nextObject ();
	c = (SafeBigInt) buffer.nextObject ();
	groupVerifier = (SafeBigInt) buffer.nextObject ();
	shareVerifier = (SafeBigInt) buffer.nextObject ();
    }

    public boolean equals (Object other) {
	Verifier rhs = (Verifier) other;
	return ((this.id == rhs.id) &&
	        this.z.equals (rhs.z) &&
	        this.c.equals (rhs.c) &&
	        this.groupVerifier.equals (rhs.groupVerifier) &&
	        this.shareVerifier.equals (rhs.shareVerifier));
    }



    public void serialize (OutputBuffer buffer) {
	buffer.add (id);
	buffer.add (z);
	buffer.add (c);
	buffer.add (groupVerifier);
	buffer.add (shareVerifier);
    }
}

