/**
 * TapestryPrefixRouteMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: TapestryPrefixRouteMsg.java,v 1.7 2002/07/23 22:36:45 hling Exp $
 */
package ostore.tapestry.api;
import ostore.tapestry.channel.MsgQos;
import ostore.util.SecureHash;
import ostore.util.QuickSerializable;
import seda.sandStorm.api.QueueElementIF;

/**
 * Messages sent from one OceanStore machine to some other, hopefully with
 * a node GUID that closely matches the given one.  The desired GUID to be
 * matched is placed in {@link #peer} when <code>! inbound</code>.  Because
 * the recipient of this message is unknown to the sender, it cannot be
 * MAC'ed.
 *
 * <p> Implementation-wise, this is simply a route-to-root operation which
 * is received by the node closest to the (likely non-existent) root.  If
 * the root exists, it is equivalent to a {@link TapestryRouteMsg} with
 * <code>mac == false</code>.
 */
public abstract class TapestryPrefixRouteMsg 
implements QueueElementIF, QuickSerializable {

    /** 
     * If <code>inbound</code>, the sender; if <code>! inbound</code> the
     * GUID to try and match.
     */
    public SecureHash peer;

    /**
     * Whether this message is being received (<code>true</code>) or sent
     * (<code>false</code>).
     */
    public boolean inbound;

    /**
     * Number of hops taken by the message on its way to the application
     */
    public int hopCount;

    /**
     * Number of hops this message should take before delivering payload.
     */
    public int TTL;

    /**
     * The delivery qos of this msg, added by Ling Huang
     */
    public int qos;

    /**
     * Construct according to the given fields.
     */
    public TapestryPrefixRouteMsg (SecureHash peer, boolean inbound) {

	this.peer = peer;
	this.inbound = inbound;
	this.qos = MsgQos.UNRELIABLE;
    }

    /**
     * Empty constructor.  For use by derived types when reconstructing
     * themselves from a byte array.
     */
    public TapestryPrefixRouteMsg () 
    {
	this.qos = MsgQos.UNRELIABLE;
    }
}








