/**
 * TapestryUnpublishMsg
 *
 * @author     Ben Y. Zhao
 * @version    $Id: TapestryUnpublishMsg.java,v 1.6 2002/10/24 21:57:13 strib Exp $
 */

package ostore.tapestry.api;
import ostore.tapestry.channel.MsgQos;
import ostore.tapestry.impl.Router;
import ostore.util.SecureHash;
import ostore.util.QuickSerializable;
import seda.sandStorm.api.QueueElementIF;

/**
 * Event that causes Tapestry to remove published backpointers to the given
 * GUID with the given tag.  
 *
 * @see TapestryLocateMsg
 * @see TapestryPublishMsg
 */
public class TapestryUnpublishMsg implements QueueElementIF {

    /**
     * The GUID.
     */
    public SecureHash guid;

    /**
     * The tag.
     */
    public TapestryTag tag;

    /**
     * The delivery qos of this msg, added by Ling Huang
     */
    public int qos;

    /**
     * A bitmask indicating which local optimizations to use.
     * Bits indicate the following optimizations, from least significant bit
     *   to most significant bit:
     *   Bit 1) Publish along secondary links, for a few hops
     *   Bit 2) Publish along all links at the current level, for a few hops
     *   Bit 3) Before leaving a LAN, misroute and publish.
     * Note: "a few hops" is defined by Router.LOCAL_OPT_HOPS
     */
    public int local_opts;

    /**
     * Construct according to the given fields.
     */
    public TapestryUnpublishMsg (SecureHash guid, TapestryTag tag) {
	this.guid = guid;
	this.tag = tag;
	this.qos = MsgQos.UNRELIABLE;
	// default: no local optimizations
	local_opts = 0;
    }

    public void setLocalOptPublishBackup() {
	local_opts = local_opts | Router.LOCAL_OPT_PUBLISH_BACKUP;
    }

    public void setLocalOptPublishAllLinks() {
	local_opts = local_opts | Router.LOCAL_OPT_PUBLISH_ALL_LINKS;
    }

    public void setLocalOptPublishLocalMisroute() {
	local_opts = local_opts | Router.LOCAL_OPT_PUBLISH_LOCAL_MISROUTE;
    }

    /**
     * Returns a human-readable representation of this
     * <code>TapestryUnpublishMsg</code>. */
    public String toString()
    {
      String str = new String();

      str += "<TapestryUnpublishMsg";
      str += " guid==" + guid;
      str += " tag==" + tag;
      str += ">";
      
      return str;
    }
}

