/**
 * ChannelHandler class.
 *
 * @author     Ling Huang
 * @version    $Id: ChannelHandler.java,v 1.20 2003/03/17 03:57:45 ravenben Exp $
 */

package ostore.tapestry.impl;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Collection;
import java.util.Iterator;
import ostore.tapestry.api.*;
import ostore.dispatch.Classifier;
import ostore.tapestry.channel.*;
import ostore.network.NetworkMessage;
import ostore.util.AssertionViolatedException;
import ostore.util.ByteUtils;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.core.ssTimer;
import ostore.util.CountBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QSArray;
import ostore.util.QSByteArray;
import ostore.util.QSException;
import seda.sandStorm.api.SinkException;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.oil.Model;

public class ChannelHandler
{
  //For debugging
    private String class_tag = "ChannelHandler";
    //    private static final boolean DEBUG = DebugFlags.TAPESTRY_CHANNEL;
    private static final boolean DEBUG = false;
    //    private static final boolean DEBUG = true;
    public  LinkedList _dispatched;

  //For functionality
    private HashMap _channels;

    private Router  _router;
    public  static final int MAX_CHANNEL_NUMBER  = 65535;

    private RetransmissionCheckMsg _check_msg;
    private boolean _check_on;

    public ChannelHandler(Router router)  
    {
        _dispatched = new LinkedList();

	_router = router;
	_channels = new HashMap();

	//Create a check msg fro retransmission check.
	_check_msg = new RetransmissionCheckMsg();
	_check_on  = false;
    }

    protected void dispatch (QueueElementIF item) 
    {
	final String tag = class_tag + ".dispatch(QueueElementIF)";

	//Added by Ling Huang to handle reliable message.
	if (item instanceof RouteMsg)
	{
	    RouteMsg msg = (RouteMsg)item;
	    if ( msg.inbound == false && msg.qos == MsgQos.RELIABLE )
	    {
		dispatch((NetworkMessage)msg, msg.get_msg_id());
		return;
	    }
	}
	else if (item instanceof PublishMsg)
	{
	    PublishMsg msg = (PublishMsg)item;
	    if ( msg.inbound == false && msg.qos == MsgQos.RELIABLE )
	    {
		dispatch((NetworkMessage)msg, msg.get_msg_id());
		return;
	    }
	}
	else if (item instanceof RouteObjRootMsg)
	{
	    RouteObjRootMsg msg = (RouteObjRootMsg)item;
	    if ( msg.inbound == false && msg.qos == MsgQos.RELIABLE )
	    {
		dispatch((NetworkMessage)msg, msg.get_msg_id());
		return;
	    }
	}
	else if (item instanceof RouteObjDestMsg)
	{
	    RouteObjDestMsg msg = (RouteObjDestMsg)item;
	    if ( msg.inbound == false && msg.qos == MsgQos.RELIABLE )
	    {
		dispatch((NetworkMessage)msg, msg.get_msg_id());
		return;
	    }
	}

	if (DEBUG) 
	    Debug.printtagln (tag, "For unreliable msg, send directly! ");
	try 
	{
	    Classifier.getClassifier(_router.get_self_node_id()).dispatch(item);
	} 
	catch (SinkException e) 
	{
	    _router.BUG (class_tag + ".dispatch: could not dispatch " + 
		    item + " exception was " + e);
	}
    }

    protected void dispatch (NetworkMessage msg, SecureHash msg_id) 
    {
	final String tag = class_tag + ".dispatch(Reliable NetworkMessage)";

	//Modified by Ling Huang: add channel in Tapestry router.
	//Channel will handle all reliable dispatch. 
	Channel  channel;
	
	if (DEBUG) 
	    Debug.printtagln (tag, "For reliable msg, send through channel! ");
	channel = get_channel(msg.peer);
	channel.send(msg, msg_id);

	//A new packet put in channel as outstanding,
	//need to do retransmission check.
	if (_check_on == false)
	{
	    _check_on = true;
	    Classifier.getClassifier(_router.get_self_node_id()).dispatch_later
		(_check_msg, Channel.DEFAULT_RETRANS_CHECK_TIME);
	    long time = System.currentTimeMillis();
	    if (DEBUG) 
		Debug.printtagln (tag, "A new packet with msg_id " + 
		msg_id + " comes in channel to turn on the " +
                "retransmission check at time " + time); 
	}
    }

    public void send_ack (NodeId prevhop, SecureHash msg_id)
    {
	//Send a ack message to previous hop.
	final String tag = class_tag + ".send_ack";
	ChannelACKMsg  ack = new ChannelACKMsg(prevhop,msg_id);

	try 
	{
	    Classifier.getClassifier(_router.get_self_node_id()).dispatch(ack);
	} 
	catch (SinkException e) 
	{
	    _router.BUG (class_tag + ".dispatch: could not dispatch " + ack);
	}

	long time = System.currentTimeMillis();
	if (DEBUG) 
	    Debug.printtagln (tag, "send an ack to " + ack.peer +
			      " at " + _router.get_self_node_id() + 
			      " for msg " + msg_id + 
			      " at time " + time);
    } 

    public void handle_msg(ChannelACKMsg msg)
    {
	Channel channel;
	final String tag = class_tag + ".handle_msg(ChannelACKMsg)";

	//Check whether the channel for next hop exists in this node.
	channel = (Channel)_channels.get(msg.peer);
	if ( channel == null)
	{
	    //channel doesn't exist, this is a bug.
	    if (DEBUG) Debug.printtagln (tag, "Channel for ack from " + 
					 msg.peer + " for msg " + 
					 msg.ack_id[0] +
					 " does not exist!");
	    //Looking for the message in all the channels.
	    //Normally it should not go this way.
	    Collection collection = _channels.values();
	    Iterator i = collection.iterator ();
	    while ( i.hasNext () ) 
	    {
		channel = (Channel)i.next ();
		channel.recv(msg);
	    }
	}
	else
	    channel.recv(msg);
	return;
    }

    // Added by Ling Huang for Channel operation: 06/28/2002.
    public boolean handle_msg (ChannelRequestMsg msg)
    {
	System.out.println ("ChannelHandler deal with ChannelRequestMsg.");
	System.out.println ("I don't know whether we still need it.");
	/*
	if (channel_response(msg) == ChannelResponseMsg.CHANNEL_SUCCEED)
	{
	    channel_creation(msg);
	    return true;
	}
	*/
	return false;
    }

    public boolean handle_msg (ChannelResponseMsg msg)
    {
	System.out.println ("I don't know whether we still need it.");
	/*
	if (msg.get_status() == msg.CHANNEL_SUCCEED)
	{
	    channel_creation(msg);
	    return true;
	}
	*/
	return false;
    }

    public void handle_msg(ChannelRouteMsg msg )
    {

    }

    /*
    public void handle_msg(ChannelICMPMsg msg)
    {
    }
    public void handle_msg(ChannelErrorMsg msg )
    {
    }
    */

    public Channel get_channel(NodeId nexthop)
    {
	Channel channel;
	String   nodeid;
	final String tag = class_tag + ".get_channel";

	if (DEBUG) 
	    Debug.printtagln (tag, "There are " + _channels.size() +
			      " channels exists!");
	//Check whether the channel for next hop exists in this node.
	channel = (Channel)_channels.get(nexthop);
	if ( channel == null)
	{
	    //channel doesn't exist, create a channel.
	    if (DEBUG) 
		Debug.printtagln (tag, "Channel for " + nexthop +
				  "does not exist!");
	    channel = channel_creation(nexthop);
	    //I use NodeId as the hash key directly, but actually 
	    //the content of NodeId (IP+port) uniquely defines a channel.
	    _channels.put(nexthop, channel);
	    //	    nodeid = (nexthop).toString();
	    //	    _channels.put(nodeid, channel);
	}
	else
	{
	    if (DEBUG) 
		Debug.printtagln (tag, "Channel for " + nexthop +
				  "exist and can use it directly!");
	}

	return channel;
    }

    public boolean channel_exist(NodeId nexthop)
    {
	return _channels.containsKey(nexthop);
    }

    //Channel creation for destination Router.
    public Channel channel_creation(NodeId nexthop)
    {
	Channel channel;

	channel = new Channel(nexthop,_router);
	//	_channels.put(nexthop, channel);

	final String tag = class_tag + ".channel_creation()";
	if (DEBUG) 
	    Debug.printtagln (tag,"Successfully create a " + 
			      "channel at " +
			    _router.get_self_node_id() + 
			    _router.get_self_guid() );
	return channel;
    }

    public void retransmit()
    {
	final String tag = class_tag + ".retransmit()";
	
	if (DEBUG) 
	    Debug.printtagln (tag, "Do iteration through all " + 
			      _channels.size() +
			      " channels to check for retransmission!");
	Channel channel;
	int     pkt_num;
	
	pkt_num = 0;
	Collection collection = _channels.values();
	Iterator i = collection.iterator ();
	while ( i.hasNext () ) 
	{
	    channel = (Channel)i.next ();
	    channel.retransmit();

	    pkt_num = pkt_num + channel.get_packet_number();
	}

	long time = System.currentTimeMillis();
	if (pkt_num > 0)
	{
	    if (DEBUG) Debug.printtagln (tag, "Still " + pkt_num + 
		       " packets in channel at time " + time +
		       ", dispatch a check_msg for later retrasmission!");
	    Classifier.getClassifier(_router.get_self_node_id()).dispatch_later
		(_check_msg, Channel.DEFAULT_RETRANS_CHECK_TIME);
	}
	else
	{
	    _check_on = false;
	    if (DEBUG)
	    {
	        Debug.printtagln (tag, "No packets in channel at time " +
		      time + ", turn off the retrasmission check!");
	        Debug.printtagln (tag, "Packets have been transmitted by" +
		      " this router so far:" );
	        Debug.printtagln (tag, "     Msg id         Transmit time      " +
		      "    RTO        DTO");

		ListIterator iter = _dispatched.listIterator ();
		while ( iter.hasNext () ) 
		{
		    OutstandingMsg msg;
		    msg = (OutstandingMsg)iter.next();
		    Debug.printtagln (tag, msg.msg_id + " " + msg.tran_time +
			  " " + msg.RTO + " " + msg.DTO);
		}
	    }
	}
    }

    /*
    // Added by Ling Huang for Channel operation: 06/28/2002.
    public boolean handle_msg (ChannelRequestMsg msg)
    {
	System.out.println ("ChannelHandler deal with ChannelRequestMsg.");
	System.out.println ("I don't know whether we still need it.");

	if (channel_response(msg) == ChannelResponseMsg.CHANNEL_SUCCEED)
	{
	    channel_creation(msg);
	    return true;
	}

	return false;
    }

    public boolean handle_msg (ChannelResponseMsg msg)
    {
	System.out.println ("I don't know whether we still need it.");
	if (msg.get_status() == msg.CHANNEL_SUCCEED)
	{
	    channel_creation(msg);
	    return true;
	}
	return false;
    }

    public void handle_msg(ChannelRouteMsg msg )
    {

    }
    public void handle_msg(ChannelACKMsg msg )
    {

    }

    public void handle_msg(ChannelICMPMsg msg)
    {
    }
    public void handle_msg(ChannelErrorMsg msg )
    {
    }
    public void channel_request(NodeId dest_ip, SecureHash dest)
    {
	ChannelRequestMsg request;
	Integer    id;
	
	id = generate_channel_id();
	while ( channel_id_exist(id) )
	    id = generate_channel_id();

	request = new ChannelRequestMsg(dest_ip, _router.get_self_guid(), 
					dest, id.intValue());
	request.inbound = false;

//_user_data error!
TapestryRouteMsgChannel    data;
data = new TapestryRouteMsgChannel(dest, MsgQos.RELIABLE);
request._user_data = (QuickSerializable)data;

	_router.dispatch(request);
    }

    public boolean channel_request(NodeId dest_ip, SecureHash dest, int channel_id)
    {
	ChannelRequestMsg request;
	Integer    id = new Integer(channel_id);
	
	if ( channel_id_exist(id) )
	    return false;

	request = new ChannelRequestMsg(dest_ip, _router.get_self_guid(), 
					dest, id.intValue());
	request.inbound = false;

//_user_data error!
TapestryRouteMsgChannel    data;
data = new TapestryRouteMsgChannel(dest, MsgQos.RELIABLE);
request._user_data = (QuickSerializable)data;

	_router.dispatch(request);
	return true;
    }

    public int channel_response(ChannelRequestMsg msg)
    {
	ChannelResponseMsg response;
	Integer    channel_id;
	
	System.out.println("Get a ChannelRequestMsg from " + msg.get_peer() + 
			   "(" + msg.get_src() + ")");

	response = new ChannelResponseMsg(msg.get_peer(), msg.get_src(), _router.get_self_guid(), 
					  msg.get_channel_id());
	response.inbound = false;

	if (_channels.size() > MAX_CHANNEL_NUMBER)
	{
	    response.set_status(response.CHANNEL_TOO_MUCH);

//_user_data error!
	    //response._user_data = response;
TapestryRouteMsgChannel    data;
data = new TapestryRouteMsgChannel( msg.get_src(), MsgQos.RELIABLE);
response._user_data = (QuickSerializable)data;

	    _router.dispatch(response);
	    return response.CHANNEL_TOO_MUCH;
	}

	channel_id = new Integer( msg.get_channel_id() );
	if ( channel_id_exist(channel_id) )
	{
	    response.set_status(response.CHANNEL_ID_EXIST);

//_user_data error!
	    //response._user_data = response;
TapestryRouteMsgChannel    data;
data = new TapestryRouteMsgChannel( msg.get_src(), MsgQos.RELIABLE);
response._user_data = (QuickSerializable)data;

	    _router.dispatch(response);
	    return response.CHANNEL_ID_EXIST;
	}

	response.set_status(response.CHANNEL_SUCCEED);

//_user_data error!
	//response._user_data = response;
TapestryRouteMsgChannel    data;
data = new TapestryRouteMsgChannel( msg.get_src(), MsgQos.RELIABLE);
response._user_data = (QuickSerializable)data;

	_router.dispatch(response);

	System.out.println("Successfully create a channel at destination end!");
	return response.CHANNEL_SUCCEED;
    }

    //Channel creation for destination Router.
    public Channel channel_creation(ChannelRequestMsg msg)
    {
	Integer    id;
	Channel    channel = null;

//	public Channel(NodeId src_id, NodeId dest_id, SecureHash src, SecureHash dest, 
	//	int channel_id)
	channel = new Channel(msg.get_peer(), _router.get_self_node_id(), 
			      msg.get_src(),  _router.get_self_guid(), 
			      msg.get_channel_id());
	id = new Integer(channel.get_channel_id());
	_channels.put(id, channel);

	System.out.println ("Successfully create a channel at the destination end!");
	return channel;
    }

    //Channel creation for source Router.
    public Channel channel_creation(ChannelResponseMsg msg)
    {
	Integer    id;
	Channel    channel = null;

//	public Channel(NodeId src_id, NodeId dest_id, SecureHash src, SecureHash dest, 
	//	int channel_id)
	channel = new Channel(_router.get_self_node_id(), msg.get_peer(), 
			      _router.get_self_guid(),    msg.get_dest(), 
			      msg.get_channel_id());
	id = new Integer(channel.get_channel_id());
	_channels.put(id, channel);

	System.out.println ("Successfully create a channel at the source end!");
	return channel;
    }


//  Functions to generate a channel_id. We currently use random number.
    public Integer generate_channel_id()
    {
	Random  random = new Random();
	int     id = random.nextInt();
	Integer channel_id = new Integer(id);
	
	return channel_id;
    }

    public boolean channel_id_exist(Integer id)
    {
	return _channels.containsKey(id);
    }
*/
}






