package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;

/**
 * DebugMsg.java
 *
 * The message that carries a request or response for a test action or
 * sequence. Sent between the FullTestDriver and any number of
 * FullTestMembers.
 *
 * @author Ben Poon
 */
public class DebugMsg 
  extends NetworkMessage implements QuickSerializable {

  /**
   * Message requesting node routing table
   */
  public static final int ROUTING_TABLE_REQUEST = 0;

  /**
   * Message requesting node back pointers
   */
  public static final int BACK_PTR_REQUEST = 1;

  /**
   * Message requesting node reverse pointers
   */
  public static final int REVERSE_PTR_REQUEST = 2;

  /**
   * Message saying that there is no more info coming
   */
  public static final int DONE = 3;

  /**
   * Message requesting information on any ongoing insertions in
   * the form of nodestate information.
   */
  public static final int INSERTION_STATE = 4;

  public int type;
  public SecureHash requestID;
  public int level; // optional

  public DebugMsg (NodeId peer, int type, SecureHash reqID, int level) {
    super (peer, false /*outbound*/);
    this.type = type;
    this.requestID = reqID;
    this.level = level;
  }

  public DebugMsg (NodeId peer, int type, SecureHash reqID) {
      super (peer, false /*outbound*/);
      this.type = type;
      this.requestID = reqID;
      this.level = -2;
  }

  public DebugMsg (InputBuffer buffer) throws QSException {
    super(buffer);
    type = buffer.nextInt();
    requestID = (SecureHash) buffer.nextObject();
    boolean has_level = buffer.nextBoolean();
    if (has_level) {
	level = buffer.nextInt();
    }
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(type);
    buffer.add(requestID);
    if (level == -2) {
	buffer.add(false);
    } else {
	buffer.add(true);
	buffer.add(level);
    }
  }

  public Object clone () throws CloneNotSupportedException {
    DebugMsg result = (DebugMsg) super.clone();
    result.type = type;
    result.requestID = requestID;
    result.level = level;
    return result;
  }

  public String toString() {
    return "(DebugMsg: type="+type+", reqID="+requestID+", to:"+peer+")";
  }

}
