/**
 * DebugRespMsg.java
 *
 * @author Ben Poon
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;

/**
 * Message used by DynamicTClient to respond to a DebugMsg
 */
public class DebugRespMsg 
  extends NetworkMessage implements QuickSerializable {

  private boolean success;
  public int requestType;
  private SecureHash requestID;
  private String output;
  private SecureHash src;

  public SecureHash rev_s;
  public NodeId rev_nid;
  public int rev_level;

  public DebugRespMsg (NodeId peer, boolean success, int requestType,
		       SecureHash requestID, SecureHash src, String output,
		       SecureHash rev_s, NodeId rev_nid, int rev_level) {
    super (peer, false /*outbound*/);
    this.success = success;
    this.requestType = requestType;
    this.requestID = requestID;
    this.src = src;
    this.output = output;
    this.rev_s = rev_s;
    this.rev_nid = rev_nid;
    this.rev_level = rev_level;
  }

    public DebugRespMsg (NodeId peer, boolean success, int requestType,
			 SecureHash requestID, SecureHash src, String output) {
	super (peer, false /*outbound*/);
	this.success = success;
	this.requestType = requestType;
	this.requestID = requestID;
	this.src = src;
	this.output = output;
	this.rev_s = null;
	this.rev_nid = null;
	this.rev_level = 0;
    }

  public DebugRespMsg (InputBuffer buffer) throws QSException {
    super(buffer);
    success = buffer.nextBoolean();
    requestType = buffer.nextInt();
    requestID = (SecureHash) buffer.nextObject();
    src = (SecureHash) buffer.nextObject();
    output = buffer.nextString();
    boolean has_rev_s = buffer.nextBoolean();
    if (has_rev_s) {
	rev_s = (SecureHash) buffer.nextObject();
    }
    boolean has_rev_nid = buffer.nextBoolean();
    if (has_rev_nid) {
        rev_nid = (NodeId) buffer.nextObject();
    }
    rev_level = buffer.nextInt();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(success);
    buffer.add(requestType);
    buffer.add(requestID);
    buffer.add(src);
    buffer.add(output);
    if (rev_s == null) {
	buffer.add(false);
    } else {
	buffer.add(true);
	buffer.add(rev_s);
    }
    if (rev_nid == null) {
	buffer.add(false);
    } else {
	buffer.add(true);
	buffer.add(rev_nid);
    }
    buffer.add(rev_level);
  }

  public SecureHash getID() {
    return requestID;
  }

  public boolean success() {
    return success;
  }

  public Object clone () throws CloneNotSupportedException {
    DebugRespMsg result = (DebugRespMsg) super.clone();
    result.success = success;
    result.requestType = requestType;
    result.requestID = requestID;
    result.output = output;
    result.src = src;
    result.rev_s = rev_s;
    result.rev_nid = rev_nid;
    result.rev_level = rev_level;
    return result;
  }

  public String toString() {
    if (success)
      return "(TestRespMsg: success on operation:"+requestType+
	", from="+peer+", id="+requestID+", output="+output +")";
    else 
      return "(TestRespMsg: failure on operation:"+requestType+
	", from="+peer+", id="+requestID+", output="+output +")";
  }
}
