/**
 * DynamicEndDelete.java
 *
 * A message that is sent by a voluntarily detaching node to nodes on its
 * backpointer list to indicate that it is all ready to be disconnected. 
 * This really means that a multicast from A has finished, and all objects
 * rooted at A has found a new root node.  When this is received, each
 * backpointer node sends it back with the message type turned to confirm.
 * When the node receives this confirmation, it actually leaves.
 * @author Ben Y. Zhao
 * @version $Id: DynamicEndDelete.java,v 1.1 2002/09/05 23:26:56 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;

public class DynamicEndDelete 
  extends NetworkMessage implements QuickSerializable {

  public SecureHash src;  //source of message
  public boolean type;     //is this request(0) or confirmation(1)
  public static final boolean CONFIRM_TYPE = true;
  public static final boolean REQUEST_TYPE = false;

  public DynamicEndDelete (NodeId peer, SecureHash src, boolean type) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.type = type;
  }

  public DynamicEndDelete (InputBuffer buffer) throws QSException {
    super(buffer);
    src = (SecureHash) buffer.nextObject();
    type = buffer.nextBoolean();
   }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(src);
    buffer.add(type);
   }

  public Object clone () throws CloneNotSupportedException {
    DynamicEndDelete result = (DynamicEndDelete) super.clone();
    result.src = src;
    result.type = type;
    return result;
  }

  public String toString() {
    if (type != CONFIRM_TYPE)
      return "(DynamicEndDelete: root-->"+src+", from/to-->"+peer+", REQUEST)";
    else 
      return "(DynamicEndDelete: root-->"+src+", from/to-->"+peer+", CONFIRM)";
  }

}
