/**
 * DynamicMCReqMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicMCReqMsg.java,v 1.12 2003/05/08 02:03:59 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.NonceMsg;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;

public class DynamicMCReqMsg 
extends NonceMsg implements QuickSerializable {

  public SecureHash src;  //source of message
  public boolean type;     //is this request(0) or confirmation(1)
  public int insertinglevel;
  public static final boolean CONFIRM_TYPE = true;
  public static final boolean REQUEST_TYPE = false;

  public DynamicMCReqMsg (NodeId peer, SecureHash src, boolean type,
			  int level) {
    super (peer);
    this.src = src;
    this.type = type;
    this.insertinglevel = level;
  }


  public DynamicMCReqMsg (InputBuffer buffer) throws QSException {
    super(buffer);
    src = (SecureHash) buffer.nextObject();
    type = buffer.nextBoolean();
    insertinglevel = buffer.nextInt();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(src);
    buffer.add(type);
    buffer.add(insertinglevel);
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicMCReqMsg result = (DynamicMCReqMsg) super.clone();
    result.src = src;
    result.type = type;
    result.insertinglevel = insertinglevel;
    return result;
  }

  public String toString() {
    if (type != CONFIRM_TYPE)
      return "(DynamicMCReqMsg: root-->"+src+", level="+insertinglevel
	+", from/to-->"+peer+", REQUEST)";
    else 
      return "(DynamicMCReqMsg: root-->"+src+", level="+insertinglevel
	+", from/to-->"+peer+", CONFIRM)";
  }

}
