/**
 * GuidAddr.java
 *
 * Class to encapsulate the GUID-NodeID pair that is often used in Tapestry.
 * This is the basic class that gets shipped around in lots of DynamicTClient
 * messages.
 * 
 * @author     Ben Y. Zhao
 * @version    $Id: GuidAddr.java,v 1.5 2003/06/05 19:15:21 jlee81 Exp $
 */

package ostore.tapestry.impl;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;

public class GuidAddr implements QuickSerializable, Cloneable, Comparable
{
  private SecureHash guid;
  private NodeId addr;

  public GuidAddr(SecureHash guid, NodeId addr) {
    this.guid = guid;
    this.addr = addr;
  }

  public GuidAddr(InputBuffer buffer) throws QSException {
    guid = (SecureHash)buffer.nextObject();
    addr = (NodeId)buffer.nextObject();
  }

  public SecureHash getGuid() {
    return guid;
  }

  public NodeId getAddr() {
    return addr;
  }

  public void serialize(OutputBuffer buffer){
    buffer.add(guid);
    buffer.add(addr);
  }

  public boolean equals (Object other) {
    if (other == null)
      return false;
    GuidAddr rhs = (GuidAddr) other;	
    if (!guid.equals(rhs.getGuid()) || !addr.equals(rhs.getAddr()))
      return false;
    return true;
  }

  public int compareTo( Object o ) throws ClassCastException
  {
    if(o == null) {
      throw new NullPointerException();
    } else if(!(o instanceof GuidAddr) ) {
      throw new ClassCastException( "cannot compareTo non-SHA1Hash" );
    } else if(o == this) {
      return 0;
    } else {
      int rv = -1;
      GuidAddr other = (GuidAddr)o;
      if((rv = guid.compareTo(other.guid)) != 0)
	return rv;
      else 
	return addr.compareTo(addr);
    }
  }

  public Object clone () throws CloneNotSupportedException
  {
    return new GuidAddr((guid==null?null:(SecureHash)guid.clone()), 
			(addr==null?null:(NodeId)addr.clone()));
  }

  public String toString() {
    return "GuidAddr: Guid="+RoutingTable.printGuid(guid)+", Addr="+addr;
  }

  public int hashCode() {
      return(guid.hashCode() ^ addr.hashCode());
  }
}
