/**
 * PartialRepublish.java
 *
 * @author     Ben Y. Zhao
 * @version    $Id: PartialRepublish.java,v 1.2 2002/10/30 23:48:54 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.util.QSException;
import ostore.util.SecureHash;
import ostore.util.OutputBuffer;
import ostore.tapestry.api.TapestryRouteMsg;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.NodeId;
import ostore.network.NetworkMessage;

/**
 * The message sent by DynamicTClient (or another Tapestry stage) to the
 * another Tapestry node (likely only 1 overlay hop away) in order to request
 * a partial republish of a particular object.  This is generally due to
 * changes in the Tapestry mesh which require the publication information to
 * be propagated to a new publication path.  
 */

public class PartialRepublish 
  extends NetworkMessage implements QuickSerializable {

  public SecureHash objGuid;
  public SecureHash requestor;
  public NodeId suggestedBackup;

  public PartialRepublish (SecureHash objGuid, NodeId dest,
			   SecureHash requestor, NodeId backup) {
    super (dest, false /*outbound*/);
    this.objGuid = objGuid;
    this.requestor = requestor;
    this.suggestedBackup = backup;
  }

  public PartialRepublish (InputBuffer buffer) throws QSException {
    super(buffer);
    objGuid = (SecureHash) buffer.nextObject();
    requestor = (SecureHash) buffer.nextObject();
    boolean backupexists = buffer.nextBoolean();
    if (backupexists) 
      suggestedBackup = (NodeId) buffer.nextObject();
    else
      suggestedBackup = null;
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(objGuid);
    buffer.add(requestor);
    if (suggestedBackup == null) 
      buffer.add(false);
    else {
      buffer.add(true);
      buffer.add(suggestedBackup);
    }
  }

  public String toString() {
    return "(PartialRepublish for object "+objGuid+", from="+requestor+
      ", with suggested backup is "+suggestedBackup+")";
  }
}


