/*
 *  StaticTClient.java
 *
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.SortedMap;
import java.util.StringTokenizer;
import java.util.TreeMap;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.StagesInitializedSignal;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.NoSuchStageException;
import seda.sandStorm.internal.SinkProxy;
import ostore.util.Clock;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.dispatch.Classifier;
import ostore.util.Carp;
import ostore.dispatch.Filter;
import ostore.util.SecureHash;
import ostore.util.NodeId;
import ostore.util.AssertionViolatedException;
import ostore.util.Clock;
import ostore.network.patchwork.Patch;

/**
 * A Tapestry client for statically configured networks.
 *
 * @author     Sean C. Rhea
 * @version    $Id: StaticTClient.java,v 1.64 2003/05/29 01:59:47 strib Exp $
 */
public class StaticTClient implements SingleThreadedEventHandlerIF, EventHandlerIF, ProfilableIF {

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for (int i=0; i < items.length; i++)
	    handleEvent (items[i]);
    }

    public void destroy() throws Exception {
	// TODO
    }

    //  methods for ProfilableIF
    public int profileSize()
    {
	return _sink.size();
    }
    //  end methods for ProfilableIF

    private String class_tag = "StaticTClient";
    private static final boolean DEBUG = false;
    private Classifier _classifier;

    private void dispatch (QueueElementIF item) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + ".");
	try {
	    Classifier.getClassifier(_self_node_id).dispatch(item);
	} catch (SinkException e) {
	    BUG (class_tag + ".dispatch: could not dispatch " + item);
	}
    }

    private void dispatch (QueueElementIF item, long delay) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + " " +
				     delay + "ms later.");

	_classifier.dispatch_later(item, delay);
    }

    private static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in " +
		"ostore.tapestry.impl.StaticTClient.  " +
		"Please let Sean know about it ASAP.");
    }
    
    public void init (ConfigDataIF config) throws Exception {
	_self_node_id = new NodeId (config.getString ("node_id"));
	if (DEBUG) Debug.turnOn (class_tag);
	class_tag = class_tag + ".[" + _self_node_id.port () + "]";

	_classifier = Classifier.getClassifier(_self_node_id);

	final String tag = class_tag + ".init";
	_timing_map = null;
	_out_times = null;
	String _dynamic_route = config.getString("dynamic_route");
	if (_dynamic_route != null && _dynamic_route.equals("dynamic")) {
	  return;
	}
	_fed_node_id = new NodeId (config.getString ("federation"));

        _ping_delay = config.getInt("PingDelay");
        if (_ping_delay == -1)
        {
            _ping_delay = 5000;
        }

	_simulator_running = config.getBoolean("simulator_running");

	_all_pairs_pings_file = config.getString("all_pairs_pings_file");

	_remember_all_static_nodes =
	    config.getBoolean("remember_all_static_nodes");

	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	      config.getManager().getProfiler().add("ostore.tapestry.impl.StaticTClient", this);
	      SinkIF sink =
		  config.getManager().getStage(stageName).getSink();
	      _sink = new SinkProxy(sink,
			      config.getManager(),
			      config.getStage().getWrapper());
	  }
	  else
	      _sink =
		  config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	    Carp.die (tag + ": could not get own sink " + e + ".");
	}

	String [] event_types = {
	    "seda.sandStorm.api.StagesInitializedSignal",
	    "ostore.tapestry.impl.NodeGuid"
	};

	for (int i = 0; i < event_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (event_types [i])))
		BUG (tag + ": could not require type " + event_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}

	String [] inb_msg_types = {
	    "ostore.tapestry.impl.StaticAllNodesMsg",
	    "ostore.tapestry.impl.StaticBeginMsg",
	    "ostore.tapestry.impl.StaticPingMsg",
	    "ostore.tapestry.impl.StaticPangMsg",
	    "ostore.tapestry.impl.StaticPongMsg",
	    "ostore.tapestry.impl.StaticPtrMsg"
	};

	for (int i = 0; i < inb_msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (inb_msg_types [i])))
		BUG (tag + ": could not require type " + inb_msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (true)))
		BUG (tag + ": could not require inbound = true for " 
			+ inb_msg_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}

    }
	

    protected void handle_stages_init_signal () {
	final String tag = class_tag + ".handle_stages_init_signal";

	if (DEBUG) Debug.printtagln (tag,
		"processing StagesInitializedSignal." +
	        " do nothing until NodeGuid recv'd.");

    }

    protected void handle_node_guid (NodeGuid local_node_guid) {
	final String tag = class_tag + ".handle_node_guid";

	if (DEBUG) Debug.printtagln (tag,
		"processing NodeGuid" );

	_self_guid = local_node_guid.node_guid;
	_self_digits = RoutingTable.digits (_self_guid);
	_revpointers = new HashMap();

        //  PRE: Take a little nap here so that the network stage on
        //  the federation has time to finish initializing.
        if (! _simulator_running)
        {
            try
            {
                Thread.sleep(5000);
            }
            catch (InterruptedException e)
            {
                //  nothing
            }
        }

	StaticAliveMsg alive_msg = new StaticAliveMsg (
		_fed_node_id, _self_guid);
	dispatch (alive_msg);
    }

    protected void handle_static_all_nodes_msg (StaticAllNodesMsg msg) {
	final String tag = class_tag + ".handle_static_all_nodes_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticAllNodesMsg" );
	if (_my_node_map != null) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticAllNodesMsg from " + msg.peer + ". " +
		    " Ignoring it.");
		    //msg + ".  Dropping it.");
	    return;
	}

	/*
	if (! msg.peer.equals (_fed_node_id)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticAllNodesMsg: " + msg + 
		    " from someone other than the federation.  " +
		    "Dropping it.");
	    return;
	}
	*/

	_my_node_map = msg.nodes ();
	if( _out_times == null) _out_times = new HashMap ();
	if( _timing_map == null) _timing_map = new HashMap ();


	java.util.Random rand = new java.util.Random(_self_node_id.hashCode());
	if( _all_pairs_pings_file != null ) {
	    parse_all_pairs_pings( msg );
	} else {
	    for (Iterator i = _my_node_map.keySet ().iterator (); 
		 i.hasNext (); )
		{
		    long delay = 1;
		    if (! _simulator_running)
			{
			    delay = 
				(long) ((_ping_delay * rand.nextDouble()) + 1);
			}
		    
		    NodeId n = (NodeId) i.next ();
		    if (DEBUG) Debug.printtagln (tag, "n=" + n);
		    if (n.less_than (_self_node_id)) {
			StaticPingMsg ping_msg = new StaticPingMsg (n);
			long now = Clock.current_date(_self_node_id).getTime();
			_out_times.put (n, new Long( now + delay ) );
			dispatch (ping_msg, delay);
		    }
		}
	}
	
	// now if for some crazy reason we did a ping-pong-pang before
	// this message reached us, and that was the only one we have to
	// do, we're done (EXTREMELY RARE CASE)
	// also useful after reading the all pairs ping file
	if (_timing_map.size () == (_my_node_map.size () - 1)) {
	    // The timing map is complete.  Fill in our routing tables
	    // and send a ready message.

	    _out_times = null;
	    _routing_tbl = calc_routing_table ();
	    send_ptr_msgs();  
	    //now send off "I'm pointing to you" messages
	    dispatch (new StaticReadyMsg (_fed_node_id));
	}

    }

    // instead of pinging everyone, just use a file to tell us what our ping
    // times are.
    protected void parse_all_pairs_pings( StaticAllNodesMsg msg) {

	final String tag = class_tag + ".parse_all_pairs_pings";
	if (DEBUG)
	    Debug.printtagln (tag, "parsing the all_pairs_pings file" );

	BufferedReader br = null;
	try {
	    br = new BufferedReader( new FileReader( _all_pairs_pings_file ) );
	} catch( FileNotFoundException fnfe ) {
	    Carp.die( "Could not open all_pairs_pings_file: " + fnfe );
	}

	// first, we read in a line containing the names of all servers
	StringTokenizer serversST = null;
	try {
	    // throw away time
	    br.readLine();
	    // throw away ping count
	    br.readLine();
	    serversST = new StringTokenizer( br.readLine() );
	} catch( IOException ioe ) {
	    Carp.die( "Could not read line from all_pairs_pings file: " + 
		      ioe);
	}
	String[] servers = new String[ serversST.countTokens() ];
	// need a map to index servers to number
	HashMap server_indices = new HashMap();
	for( int i = 0; i < servers.length; i++ ) {
	    servers[i] = serversST.nextToken();
	    NodeId currNode = null;
	    try {
		currNode = new NodeId( servers[i] + ":3630" );
	    } catch( Exception e ) {
		Carp.die( "Exception creating NodeId for " + servers[i] +
			  ": " + e );
	    }
	    server_indices.put( currNode.address(), new Integer(i) );
	    if( DEBUG )
		Debug.printtagln( tag, "creating entry for " + currNode );
	}

	// now we need to find the line corresponding to our pings
	Integer bi = (Integer) server_indices.get( _self_node_id.address() );
	if( bi == null ) {
	    Carp.die( "Could not find ourselves in the all_pairs_ping file" );
	}

	int self_index = bi.intValue();
	
	StringTokenizer pingsST = null;
	try {
	    for( int j = 0; j < self_index; j++ ) {
		br.readLine();
	    }
	    pingsST = new StringTokenizer( br.readLine() );
	} catch( IOException ioe ) {
	    Carp.die( "Could not read line from all_pairs_pings file: " + 
		      ioe );
	}

	// parse out our ping times
	double pings[] = new double[ pingsST.countTokens() ];
	for( int j = 0; j < pings.length; j++ ) {
	    // read out av ping time (the one in the middle)
	    StringTokenizer slashST = new StringTokenizer( pingsST.nextToken(),
							   "/" );
	    // throw away min time
	    slashST.nextToken();
	    // now get get av time
	    pings[j] = Double.parseDouble( slashST.nextToken() );
	}

	// they gotta be the same length
	if( pings.length != servers.length ) {
	    Carp.die( "The length of the pings row (" + pings.length +
		      ") does not equal the length of the servers row (" +
		      servers.length + ")." );
	}

	// now we need to insert the latencies into the table
	NodeId currNode;
	boolean forward_next = false;
	SortedMap smap = new TreeMap();
	smap.putAll( _my_node_map );
	for (Iterator iter = smap.keySet ().iterator (); 
	     iter.hasNext (); ) {

	    currNode = (NodeId) iter.next();

	    // if this is us, we can ignore it
	    if( currNode.equals( _self_node_id ) ) {
	        forward_next = true;
		continue;
	    }

	    // forward the AllNodes msg to the next guy on the list, please
	    if( forward_next ) {
	        forward_next = false;
		msg.inbound = false;
		msg.peer = currNode;
		if (DEBUG) 
		    Debug.printtagln (tag, "forwarding StaticAllNodes to " +
				      currNode );
		dispatch( msg );
	    }

	    Integer big_i = (Integer) server_indices.get( currNode.address() );
	    if( big_i == null ) {
		Carp.die( "Could not find " + currNode + 
			  " in the all_pairs_ping file" );
	    }
	    double ping_value = pings[ big_i.intValue() ];

	    // make this into a long by the default cast method
	    // and insert as a latency
	    _timing_map.put (currNode, new Long ((long) ping_value));
	    
	    if (DEBUG) 
		Debug.printtagln (tag, "latency to " + currNode 
				  + " is " + ping_value );

	}

    }

    protected void handle_static_ping_msg (StaticPingMsg msg) {
	final String tag = class_tag + ".handle_static_ping_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPingMsg" );
	if (! _running) {
	    long now = Clock.current_date(_self_node_id).getTime();
	    if( _out_times == null ) _out_times = new HashMap();
	    _out_times.put (msg.peer, new Long (now));
	    dispatch (new StaticPongMsg (msg.peer));
	}
    }

    protected void handle_static_pong_msg (StaticPongMsg msg) {
	long recv_time = Clock.current_date(_self_node_id).getTime();

	final String tag = class_tag + ".handle_static_pong_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPongMsg" );

	if ((_out_times == null) || (_out_times.isEmpty ())) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticPongMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! _out_times.containsKey (msg.peer)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticPongMsg: " + msg + 
		    " from someone unexpected.  Dropping it.");
	    return;
	}

	if (! _running) {
	    dispatch (new StaticPangMsg (msg.peer));
	}

	round_trip_done (recv_time, msg.peer);
    }

    protected void handle_static_pang_msg (StaticPangMsg msg) {
	long recv_time = Clock.current_date(_self_node_id).getTime();

	final String tag = class_tag + ".handle_static_pang_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPangMsg" );

	if ((_out_times == null) || (_out_times.isEmpty ())) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticPangMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! _out_times.containsKey (msg.peer)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticPangMsg: " + msg + 
		    " from someone unexpected.  Dropping it.");
	    return;
	}

	round_trip_done (recv_time, msg.peer);
    }

    protected void round_trip_done (long recv_time, NodeId peer) {
	final String tag = class_tag + ".handle_static_pang_msg";

	long send_time = ((Long) _out_times.remove (peer)).longValue ();
	long delta = recv_time - send_time;

	if( _timing_map == null) {_timing_map = new HashMap ();}

	_timing_map.put (peer, new Long (delta));

	if (DEBUG && _my_node_map != null ) 
	    Debug.printtagln (tag, 
			      "_out_times.size () = " + _out_times.size() + 
			      ", _timing_map.size () = " + _timing_map.size()+ 
			      ", _my_node_map.size () = "+_my_node_map.size());

	// if some how the ping-pong-pang happened to take place before
	// we get a StaticAllNodes (in can happen in simulation I swear)
	// then we need to fake it for now
	if (_my_node_map != null && 
	    _timing_map.size () == (_my_node_map.size () - 1)) {
	    // The timing map is complete.  Fill in our routing tables
	    // and send a ready message.

	    _out_times = null;
	    _routing_tbl = calc_routing_table ();
	    send_ptr_msgs();  
	    //now send off "I'm pointing to you" messages
	    dispatch (new StaticReadyMsg (_fed_node_id));
	}

    }

    private void send_ptr_msgs() {
      NodeId dest;
      boolean empty;
      StaticPtrMsg msg;
      for (int i=0; i<RoutingTable.DIGITS_PER_GUID; i++) {
	empty = true;
	for (int j=0; j<RoutingTable.DIGIT_VALUES; j++) {
	  RouteEntry re = _routing_tbl.getEntry( i,j );
	  if( re != null ) {
	      for( int k = 0; k < re.size(); k++ ) {
		  dest = re.getAtIndex(k);
		  if (dest == null) continue;
		  empty = false;
		  if (!dest.equals(_self_node_id)) {
		      if (DEBUG)
			  Debug.printtagln("StaticTClient.send_ptr_msgs." 
					   +_self_node_id,
					   "Sending ptrmsg to "+dest+
					   " for level "+i);
		      msg = new StaticPtrMsg(dest, _self_guid, i);
		      dispatch(msg);
		  }
	      }
	  }
	}
	if (empty) break;
      }

    }

    protected void handle_static_begin_msg (StaticBeginMsg msg) 
    throws EventHandlerException {
	final String tag = class_tag + ".handle_static_begin_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticBeginMsg" );

	if (_running) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticBeginMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! msg.peer.equals (_fed_node_id)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticAllNodesMsg: " + msg + 
		    " from someone other than the federation.  " +
		    "Dropping it.");
	    return;
	}

	dispatch (new RoutingTable.Available (_routing_tbl, _revpointers,
					      new HashMap()));

	if (DEBUG) {
 	  System.out.println(tag+
			   "dispatching routing table plus revpointers: "+
			   _routing_tbl.toString());
	  Router.dumpRevpointers(_revpointers, 3);
	}
	_running = true;
	TapestryReadyMsg ready = new TapestryReadyMsg(_self_guid);
	dispatch(ready);
    }

    public void handleEvent(QueueElementIF item) 
	throws EventHandlerException {
      try
      {
        final String tag = class_tag + ".handleEvent";

	if (DEBUG) Debug.printtagln (tag, "got " + item.getClass () + 
		    " event: " + item + ".");

	if (item instanceof StagesInitializedSignal) {
	    handle_stages_init_signal ();
	}
	else if (item instanceof NodeGuid) {
	    handle_node_guid ((NodeGuid)item);
	}
	else if (item instanceof StaticAllNodesMsg) {
	    handle_static_all_nodes_msg ((StaticAllNodesMsg) item);
	}
	else if (item instanceof StaticPingMsg) {
	    handle_static_ping_msg ((StaticPingMsg) item);
	}
	else if (item instanceof StaticPongMsg) {
	    handle_static_pong_msg ((StaticPongMsg) item);
	}
	else if (item instanceof StaticPangMsg) {
	    handle_static_pang_msg ((StaticPangMsg) item);
	}
	else if (item instanceof StaticBeginMsg) {
	    handle_static_begin_msg ((StaticBeginMsg) item);
	}
	else if (item instanceof StaticPtrMsg) {
	  StaticPtrMsg msg = (StaticPtrMsg) item;
	  if (DEBUG)
	    Debug.printtagln(tag, "Adding revpointer from "+msg.srcguid
			     +" to level "+msg.level);
	  HashMap revs = (HashMap) _revpointers.get(new Integer(msg.level));
	  if (revs == null) {
	    revs = new HashMap();
	    _revpointers.put(new Integer(msg.level), revs);
	  }
	  if (!revs.containsKey(msg.srcguid)) {
	      revs.put(msg.srcguid, msg.peer);
	  }
	  //	  Router.dumpRevpointers(_revpointers, 3);
	}
      }
      catch(Exception e)
      {
	Carp.setThrowException(false);
        Carp.die(e); 
      }
    }

    protected RoutingTable calc_routing_table () {
	final String tag = class_tag + ".calc_routing_table";

	RoutingTable result = new RoutingTable (_self_node_id, _self_guid,
						_classifier);

	LinkedList [] [] candidates = 
	    new LinkedList [result.digits_per_guid ()] [];
	for (int i = 0; i < result.digits_per_guid (); ++i)
	    candidates [i] = new LinkedList [result.digit_values ()];

	// Sort all the nodes by matching prefix.

	for (Iterator i = _my_node_map.keySet ().iterator (); i.hasNext (); ) {
	    NodeId n = (NodeId) i.next ();
	    if (! n.equals (_self_node_id)) {
		SecureHash g = (SecureHash) _my_node_map.get (n);
		int [] digits = RoutingTable.digits (g);
		int digit = 0;
		for ( ; digit < result.digits_per_guid (); ++digit) 
		    if (digits [digit] != _self_digits [digit]) break;
		if (digit == result.digits_per_guid ())
		  throw new AssertionViolatedException(tag +": duplicate GUID.");

		for (int j=0; j<=digit; j++) {
		  LinkedList l = candidates [j][digits [j]];
		  if (l == null)
		    l = candidates [j][digits [j]] = new LinkedList ();
		  l.addLast (n);
		}
	    }
	}

	// Find the closest node for each matching prefix.

	for (int digit = 0; digit < result.digits_per_guid (); ++digit) {
	    for (int value = 0; value < result.digit_values (); ++value) {
		LinkedList l = candidates [digit][value];
		if (_self_digits [digit] == value) {

		    if (DEBUG) Debug.printtagln (tag, "We are our own " +
			    "neighbor for digit " + digit + ", value " +
			    Integer.toHexString (value) + ".");

		    // Add in our own GUID.
		    // if (l != null)
		    //  throw new AssertionViolatedException(tag + "Shoot!");
		    result.write (digit, value, _self_node_id,
			    (SecureHash) _my_node_map.get (_self_node_id),
				  new Double(0));
		    //now add the rest of the entries
		    if (l != null) {
		      while (!l.isEmpty()) {
			  if (DEBUG) Debug.printtagln (tag, "backup!" );
			NodeId closest = (NodeId) l.removeFirst();
			Long latency = (Long) _timing_map.get(closest);
			result.write(digit, value, closest, 
				     (SecureHash)_my_node_map.get(closest),
				     new Double(latency.doubleValue()));
		      }
		    } //end if
		}
		else if (l != null) {

		    /* old implementation (no backups)
		    NodeId closest = (NodeId) l.removeFirst ();
		    long latency = 
			((Long) _timing_map.get (closest)).longValue (); 
		    while (! l.isEmpty ()) {
			NodeId n = (NodeId) l.removeFirst ();
			long nl = ((Long) _timing_map.get (n)).longValue ();
			if (nl < latency) {
			    closest = n;
			    latency = nl;
			}
		    }
		    */

		    NodeId closest = null;
		    long latency = 0;
		    int index = 0;
		    for( int i = 0; i < l.size(); i++ ) {
			NodeId n = (NodeId) l.get( i );
			long nl = ((Long) _timing_map.get (n)).longValue ();
			if ( closest == null || nl < latency) {
			    closest = n;
			    latency = nl;
			    index = i;
			}
		    }
		    // remove the closest one
		    l.remove( index );

		    if (DEBUG) Debug.printtagln (tag, "Our " +
			    "neighbor for digit " + digit + ", value " +
			    Integer.toHexString (value) + " is " +
			    closest + ".");

		    result.write (digit, value, closest,
			    (SecureHash) _my_node_map.get (closest),
				  new Double((double) latency));

		    result.addDistance(latency, closest,
				       (SecureHash) _my_node_map.get(closest));
		    //now add the rest of the entries
		    while (!l.isEmpty()) {
		      closest = (NodeId) l.removeFirst();
		      result.write(digit, value, closest, 
				   (SecureHash) _my_node_map.get(closest),
				   new Double(((Long) 
				     _timing_map.get(closest)).doubleValue()));
		    }
		}
	    }
	}

	if( _remember_all_static_nodes ) {
	    // Make sure _all_ static nodes are in routing table
	    // distance map, even those not used in our routing table:
	    Iterator node_iter = _my_node_map.keySet().iterator();
	    while( node_iter.hasNext() ) {
		NodeId node_id = (NodeId)node_iter.next();
		SecureHash node_guid = (SecureHash)_my_node_map.get( node_id );
		double distance;
		if( node_id.equals( _self_node_id ) ) {
		    distance = 0;
		} else {
		    distance = ((Long)_timing_map.get( node_id)).doubleValue();
		}
		result.addDistance( distance, node_id, node_guid );
	    }
	}
	
	// clear out removed_node, since we won't need them
	// (we haven't sent out any PtrMsgs yet)
	result.get_removed_nodes().clear();

	return result;
    }

    private NodeId _fed_node_id;
    private SinkIF _sink;
    private boolean _running;
    private SecureHash _self_guid;
    private int [] _self_digits;
    private Map _timing_map;
    private Map _out_times;
    private RoutingTable _routing_tbl;
    private HashMap _revpointers;
    private Map _my_node_map;
    private NodeId _self_node_id;
    private int _ping_delay;
    private boolean _simulator_running;
    private String _all_pairs_pings_file;
    private boolean _remember_all_static_nodes;
}

