/**
 * TestClient.java
 *
 *  @author     Ben Y. Zhao
 *  @version    $Id: TestShuttle.java,v 1.3 2003/02/19 10:32:31 hweather Exp $
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import java.util.*;
import java.io.*;
import java.security.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.tapestry.*;
import ostore.tapestry.api.*;
import ostore.dataobj.*;
import ostore.update.*;
import ostore.security.*;
import ostore.security.thresh.*;
import ostore.startrek.*;
import seda.sandStorm.api.*;
import seda.sandStorm.core.*;
import seda.sandStorm.internal.SinkProxy;
import cryptix.jce.provider.random.DevRandom;

public class TestShuttle implements EventHandlerIF, ProfilableIF {

    public String class_tag = "Test";

    private void dispatch (QueueElementIF item) {
	try {
	    _classifier.dispatch(item);
	} catch (SinkException e) {
	    Carp.out ("Could not dispatch " + item);
	    System.exit (1);
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    public void printtagln(String tag, String msg) {
        System.err.print(tag+":"+msg+"\n");
    }

    public void init (ConfigDataIF config) throws Exception {

	DEBUG = true;

	Types.registerAllTypes ();

	_classifier = Classifier.getClassifier (
                new NodeId (config.getString ("node_id")));
	
	//initialize the numerical arguments
	_oldobjs = Integer.parseInt(config.getString ("oldobjs"));
	_numid = config.getString ("nameindex");
	_id = Integer.parseInt(_numid);

	_self_node_id = new NodeId (config.getString ("node_id"));

	class_tag = class_tag + ".["+_self_node_id.port()+"]";
	if (DEBUG) ostore.util.Debug.turnOn(class_tag);

	final String tag = class_tag +".init";
	String on = config.getString("power");
	if (on != null && on.equalsIgnoreCase("off")) return;

	//generate GUIDs for all clients
	SecureHash guidx;
	_guid_located = new HashMap();
	for (int i=0; i<_oldobjs; i++) {
	  guidx = new SHA1Hash ((_numid+i).getBytes());
	  _guid_located.put(guidx, Boolean.FALSE);
	}
	if (DEBUG) printtagln(tag, "Done initializing guid hashmaps");

	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	    config.getManager().getProfiler().add("ostore.tapestry.impl.TestShuttle", this);
	    SinkIF sink =
	      config.getManager().getStage(stageName).getSink();
	    this_sink = new SinkProxy(sink,
				      config.getManager(),
				      config.getStage().getWrapper());
	  }
	  else
	    this_sink =
	      config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	  Carp.die (tag + ": could not get own sink.");
	}

	Filter filter;
	String [] event_types = {
	  "seda.sandStorm.api.StagesInitializedSignal",
	  "ostore.tapestry.impl.TapestryReadyMsg"
	};	  

	for (int i=0; i< event_types.length; i++) {
	  filter = new Filter ();
	  if (! filter.requireType (Class.forName(event_types[i]))) {
	    Carp.out ("Could not require type " + event_types[i]);
	    System.exit (1);
	  }
	  _classifier.subscribe (filter, this_sink);
	}

	String [] msg_types = {
	  "ostore.tapestry.impl.TestLocateMsg"
	};

	for (int i=0; i< msg_types.length; i++) {
	  filter = new Filter ();
	  if (! filter.requireType (Class.forName(msg_types[i])))
	    Carp.die ("Could not require type " + msg_types[i]);
	  if (! filter.requireValue("inbound", new Boolean(true)))
	    Carp.die ("Could not require inbound = true for "+
		      msg_types[i]);
	  _classifier.subscribe (filter, this_sink);
	}

	System.out.println("TestShuttle Tapestry stage initialized!!");
    }

  public void handleEvent(QueueElementIF item) 
    throws EventHandlerException {

    DEBUG = true;
    final String tag = class_tag+".handleEvent";

    if (item instanceof StagesInitializedSignal) {
      // do nothing
      
    } else if (item instanceof TapestryReadyMsg) {

      SecureHash objguid;
      TapestryPublishMsg msg;
      _client_guid = ((TapestryReadyMsg)item).node_guid;
      for (Iterator i=_guid_located.keySet().iterator(); i.hasNext();) {
	objguid = (SecureHash) i.next();
	msg = new TapestryPublishMsg(objguid, null);
	if (DEBUG) printtagln(tag, "Publishing " + printGuid(objguid)
			      + ".");
	dispatch(msg);
      }
    } else if (item instanceof TestLocateMsg) {
      
      TestLocateMsg msg = (TestLocateMsg) item;
      if (DEBUG) printtagln(tag, "Got TestLocateMsg from "+
			    msg.peer+" for guid: "+
			    printGuid(msg.guid)+" w/ hop="+
			    msg.hopCount);
      //now send back a routemsg to the originator of the locatemsg
      TestFoundMsg outmsg = new TestFoundMsg(msg.peer, msg.guid);
      dispatch(outmsg);
    } else {
      Carp.die ("ostore.tapestry.impl.Test: Received unknown "
		+ "QueueElementIF: " + item);
    }
  }
  
  public void destroy() throws Exception {
    // TODO
  }

    //  methods for ProfilableIF
    /* Specified by seda.sandStorm.api.ProfilableIF */
    public int profileSize()
    {
	return this_sink.size();
    }
    //  end methods for ProfilableIF

    public static String printGuid (SecureHash guid) {
        String out="";
	int [] gdigits = RoutingTable.digits(guid);
	for (int i = 0; i < gdigits.length; ++i) {
	    out += Integer.toHexString (gdigits [i]);
	}   
	return out;
    }

  private boolean DEBUG;
  private String _numid;
  private int _id;
  private int _oldobjs;
  private SecureHash _client_guid;
  
  private HashMap _guid_located;  //guid->bool: done locatemsg on them
  private SinkIF this_sink;

  private NodeId _self_node_id;
  private Classifier _classifier;

}

