/**
 * PatchworkSimulationStage.java
 *
 * @author	Jeremy Stribling
 * @version	$Id: PatchworkSimulationStage.java,v 1.24 2003/04/24 23:35:09 strib Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ostore.tapestry.patchwork;

import ostore.util.InvalidConfigFileException;

import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Resender;
import ostore.dispatch.Signal;
import ostore.network.CloseLocalPortMsg;
import ostore.network.patchwork.InjectFaultMsg;
import ostore.network.patchwork.NodeStatusMsg;
import ostore.network.patchwork.MonitorAllMsg;
import ostore.tapestry.api.TapestryNodeDetachMsg;
import ostore.tapestry.api.TapestryDetachConfirm;
import ostore.tapestry.api.TapestryIntegrateMsg;
import ostore.tapestry.impl.FullTestMsg;
import ostore.tapestry.impl.Router;
import ostore.tapestry.impl.RoutingTable;
import ostore.tapestry.impl.TapestryReadyMsg;
import ostore.tapestry.impl.VerifyRespMsg;
import ostore.util.Clock;
import ostore.util.DestroyStages;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.SHA1Hash;
import ostore.util.TypeTable;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.UnknownEventException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.ManagerIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.SinkFullException;   
import seda.sandStorm.api.StagesInitializedSignal; 
import java.util.Random;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

/**
 * @author 	Jeremy Stribling
 * @version	$Id: PatchworkSimulationStage.java,v 1.24 2003/04/24 23:35:09 strib Exp $
 */  

public class PatchworkSimulationStage implements EventHandlerIF
{
    ////////////////////       CONSTANTS	////////////////////

    private static final boolean DEBUG = true;
    private Classifier classifier;

    private SinkIF my_sink;
    private RoutingTable routing_table;
    private HashMap revpointers;

    private NodeId self_node_id;
    private SecureHash self_guid;
    private NodeId repairMonitor;

    private boolean sim_running;
    // whether or not the repair monitor can send you faults
    private boolean deletable;
    private boolean fault_injected;

    private ManagerIF manager;

    private Resender resender;
    private Object resend_token;

    ////////////////////  STATIC CONSTRUCTORS	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////
    ////////////////////     STATIC METHODS 	////////////////////
    //////////////////// 	METHODS		////////////////////

    /**
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void handleEvents(QueueElementIF elemarr[]) 
	throws EventHandlerException
    {

	for( int i=0; i<elemarr.length; i++ ) {
	    handleEvent( elemarr[i] );
	}
    }


    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void handleEvent(QueueElementIF element) 
	throws EventHandlerException
    {

	//printMsg( "Received event " + element );

      try
      {
	if( element instanceof StagesInitializedSignal ) {

	    dispatch( new MonitorAllMsg( new Object(), my_sink ) );

        } else if( element instanceof TapestryReadyMsg ) {

	    self_guid = ((TapestryReadyMsg) element).node_guid;
	    printMsg( "Joined the network" );
	    if( repairMonitor != null ) {
		dispatch( new RepairMonitorStage.AttachMessage(repairMonitor,
							       self_guid));
	    } else {
		// Find the repair monitor, try resending for a minute
		SecureHash rmHash = new SHA1Hash( "RepairMonitor" );
		resend_token = resender.resend( new RMLocateMsg( rmHash, 
								 self_node_id, 
								 self_guid,
								 deletable ),
						12, 5000 );
	    }


        } else if( element instanceof RoutingTable.Available ) {
	
	    RoutingTable.Available rta = (RoutingTable.Available) element;
	    routing_table = rta.table;
	    revpointers = rta.revpointers;
	    printMsg( "got routing table, guid: " + self_guid );
	    printMsg( "RoutingTable: \n" + routing_table );

	} else if( element instanceof InjectFaultMsg ) {

	    InjectFaultMsg pfm = (InjectFaultMsg) element;

	    // if we haven't received the RMFound, use the peer as the RM
	    if( repairMonitor == null ) {
		repairMonitor = pfm.peer;
	    }
	    
	    // sanity check -- make sure its the RM
	    if( !repairMonitor.equals( pfm.peer ) ) {
		printMsg( "WARNING! Got a Fault from someone (" + pfm.peer
			  + ") who's not the RepairMonitor (" + repairMonitor
			  + ").  Ignoring." );
		return;
	    }

	    fault_injected = true;
	    int revSize = 0;
	    int levels = 0;

	    // if we don't have revpointers, we might be using tapas
	    // so just send back a -1 and let the RM sort it out
	    if( revpointers == null ) {
		dispatch( new PatchworkRevCountMsg( repairMonitor, -1 ) );
		return;
	    }

	    // let the fault injector know how many neighbors I have
	    for (Iterator i=revpointers.entrySet().iterator(); i.hasNext(); ) {
		Map.Entry me = (Map.Entry) i.next();
		HashMap level = (HashMap) me.getValue();
		int levelnum = ((Integer) me.getKey()).intValue();
		//revSize += level.size();
		// only count the rev pointers if they match in the right
		// number of digits, so that we don't count backups
		for( Iterator j=level.keySet().iterator(); j.hasNext(); ) {
		    
		    if( Router.guidCompare( (SecureHash) j.next(),
		    		    self_guid ) == levelnum ) {
			revSize++;
		    }
		}
		levels++;
	       
	    }
	    printMsg( "Dumping rev pointers" );
	    ostore.tapestry.impl.Router.dumpRevpointers( revpointers, 5 );
	    dispatch( new PatchworkRevCountMsg( repairMonitor, revSize ) );

	} else if( element instanceof RMFoundMsg ) {

	    RMFoundMsg rmfm = (RMFoundMsg) element;

	    // we should only ever get one of these
	    if( repairMonitor != null ) {
		printMsg( "WARNING!  Got duplicate RMFound.  Ignoring." );
		return;
	    }
	    
	    repairMonitor = rmfm.peer;
	    if( resend_token != null ) {
		resender.cancel( resend_token );
	    }

	} else if( element instanceof Signal.AlarmSignal ) {

	    resender.process_queues();

	} else if( element instanceof FullTestMsg ) {

	    FullTestMsg ftm = (FullTestMsg) element;

	    if( ftm.type != FullTestMsg.VERIFY_REQUEST ) {
		printMsg( "Got a FullTestMsg for a type I don't understand: "
			  + ftm.type + ". Ignoring." );
		return;
	    }

	    if( routing_table != null ) {
		// send back a VerifyMsg
		VerifyRespMsg vm = new VerifyRespMsg( ftm.peer, true, ftm.type,
						      ftm.requestID, self_guid,
						      "No output", 
						      routing_table,
						      revpointers );
		dispatch( vm );

	    } else {
		
		printMsg( "WARNING!  Someone tried to get my RoutingTable " +
			  "when I didn't have one. Ignoring." );
		return;

	    }

	} else if( element instanceof RepairMonitorStage.AttachMessage ) {
	    
	    RepairMonitorStage.AttachMessage ac = 
		(RepairMonitorStage.AttachMessage) element;

	    // ignore if not a command
	    if( !ac.command ) {
		return;
	    }

	    // we're being told to reintegrate, so go for it
	    // we need to make ourselves a new guid before reintegration starts
	    if( ac.new_guid ) {
		self_guid.hash(self_guid);
	    }
	    printMsg( "reintegrating" );
	    dispatch( new TapestryIntegrateMsg() );

	} else if( element instanceof NodeStatusMsg ) {

	    NodeStatusMsg pnsm = (NodeStatusMsg) element;

	    // don't worry about this if you're faulty
	    if( fault_injected ) {
		return;
	    }

	    // Hakim's crazy sanity checks
	    if( repairMonitor==null || 
		repairMonitor.address()==null || 
		repairMonitor.port()==-1 || pnsm.peer==null)
	    {
		Carp.out("PatchworkSimulationStage.handleEvent" +
			 "[" + self_node_id + "]" +
			 " instanceof NodeStatusMsg" +
			 " pnsm" + pnsm +
			 " pnsm.node=" + (pnsm==null?null:pnsm.peer) +
			 " repairMonitor=" + repairMonitor +

			 " repairMonitor.address=" + 
			 (repairMonitor==null?null:repairMonitor.address()) +
			 " repairMonitor.port=" + 
			 (repairMonitor==null?-1:repairMonitor.port()));
		return;
	    }

	    printMsg( "sending ptrdeleted" );

	    // must figure out the guid and the hops ourselves
	    int hops = -1;
	    if( routing_table != null ) {
		SecureHash guid = routing_table.node_id_to_guid( pnsm.peer );
		if( guid != null ) {
		    hops = Router.guidCompare( guid, self_guid );
		}
	    }
	    dispatch( new RepairMonitorStage.PtrDeletedMsg( repairMonitor,
							    pnsm.peer, hops) );


	} else if( element instanceof RepairMonitorStage.DetachMessage ) {

	    RepairMonitorStage.DetachMessage dm = 
		(RepairMonitorStage.DetachMessage) element;

	    // if it's not a command, ignore it
	    if( !dm.command ) {
		printMsg( "Got a DetachMessage, but it's not a command " +
			  "so ignoring." );
		return;
	    }

	    /*
	    // make sure we're not already detached
	    if( routing_table == null ) {
		printMsg( "Got a DetachMessage, but we're already " + 
			  "detached.  Ignoring" );
		return;
	    }
	    */

	    // detach immediately
	    printMsg( "detaching now" );
	    //dispatch( new TapestryNodeDetachMsg() );
	    dispatch( new TapestryDetachConfirm() );

	} else if( element instanceof TapestryDetachConfirm ) {

	    printMsg( "finished detachment" );
	    routing_table = null;
	    revpointers = null;

	    // hmmm, can we be sure, in non-simulation, that this 
	    // will be dispatched? The Network thread could be stopped.
	    // Actually, with YANetwork it's probably ok, otherwise
	    // it's a bit sketchy.
	    dispatch( new RepairMonitorStage.DetachMessage( repairMonitor, 
							    false ) );

	    // actually destroy this node completely, because crash only
	    // software rocks.
	    if( sim_running ) {
		// in the simulator it's easy, just call removeNode
		soss.core.Main.removeNode( self_node_id );
	    } else {

		// in SandStorm it's harder.  Here's what you do:
		// 1) Remove the classifier for this node, so that
		//    various references can be deleted and a new
		//    node can start later with a new classifier
		Classifier.removeClassifier( self_node_id );
		// 2) Tell the Network to stop listening on this port
		dispatch( new CloseLocalPortMsg( self_node_id.port() ) );
		// 3) Fork a thread off that will destroy all stages
		//    (including this one) that belong to this node.
		//    If you try to destroy this stage from itself,
		//    you could end up incompletely destroying
		//    the state and leaving all sorts of references behind.
		DestroyStages ds = new DestroyStages( manager, self_node_id );
		Thread t = new Thread( ds );
		t.setPriority( Thread.MAX_PRIORITY );
		// this call to start has to be the last thing done,
		// because there's no guarantee we'll be allowed to go
		// any farther, the thread might be stopped.
		t.start();
		
	    }

	} else {

	  Carp.error( "Dropping unexpected message: " + element );

	}

      }
      catch(Exception e)
      {
        Carp.dieGracefully(e, self_node_id); 
      }
    }

    private void printMsg( String msg ) {
	if( DEBUG ) {
	    System.err.println( "[" + self_node_id + "]: " + msg );
	}
    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void init(ConfigDataIF config) throws Exception
    {

	self_node_id = new NodeId (config.getString ("node_id"));
	sim_running = config.getBoolean( "simulator_running" );
	deletable = config.getBoolean( "deletable" );
	manager = config.getManager();

	//////////////////
	// initialize filters

	classifier = Classifier.getClassifier( 
			new NodeId( config.getString( "node_id" ) ) );
	my_sink = config.getStage().getSink();
	Filter filter;

	// Subscribe to all my favorite events

	String[] eventNames = {
	    "seda.sandStorm.api.StagesInitializedSignal",	
	    "ostore.tapestry.api.TapestryDetachConfirm",
	    "ostore.tapestry.impl.RoutingTable$Available",
	    "ostore.network.patchwork.NodeStatusMsg",
	    "ostore.tapestry.impl.TapestryReadyMsg"
	};

	for( int i = 0; i < eventNames.length; i++ ) {
	  filter = new Filter();
	  Class eventClass;
          try {
            eventClass = Class.forName( eventNames[i] );
	    if( !filter.requireType( eventClass ) || 
	        !classifier.subscribe( filter, my_sink ) ) {

	      Carp.die( "Could not require class " + eventNames[i] ); 

	    } 

          } catch ( ClassNotFoundException cnfe ) {
            Carp.die( cnfe );
          }

	}

	String[] inb_eventNames = {
	    "ostore.network.patchwork.InjectFaultMsg",
	    "ostore.tapestry.patchwork.RMFoundMsg",
	    "ostore.tapestry.impl.FullTestMsg",
	    "ostore.tapestry.patchwork.RepairMonitorStage$AttachMessage",
	    "ostore.tapestry.patchwork.RepairMonitorStage$DetachMessage"
	};

	// require inbounders
	for( int i = 0; i < inb_eventNames.length; i++ ) {
	  filter = new Filter();
	  Class eventClass;
          try {
            eventClass = Class.forName( inb_eventNames[i] );
	    if( !filter.requireType( eventClass ) ||
		!filter.requireValue ( "inbound", new Boolean(true) ) || 
	        !classifier.subscribe( filter, my_sink ) ) {

	      Carp.die( "Could not require class " + inb_eventNames[i] ); 

	    } 

          } catch ( ClassNotFoundException cnfe ) {
            Carp.die( cnfe );
          }

        }

	resender = new Resender( self_node_id, config.getStage().getName() );
	// listen for the resender events
	Signal.getSignal(self_node_id).register(Signal.ALRM, 
						config.getStage().getName(),
						my_sink);

    }

    /**
     * An internal method used for dispatching events
     */
    private void dispatch( QueueElementIF event ) {

	//printMsg( "Dispatching " + event );
      try {
        classifier.dispatch( event );
      } catch ( SinkFullException sfe ) {
        //TODO: Better way to deal with SinkFull
        try {
          Thread.currentThread().sleep( 5000 );
        } catch( InterruptedException ie ) {
          //ignore
        }
      }                                         
      
    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void destroy()
    {
    }

}
