/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;
import java.util.*;

/**
 * Debug provides printing facilities for debug purposes.
 *
 * @author     Sean C. Rhea
 * @version    $Id: Debug.java,v 1.10 2002/08/07 04:20:53 srhea Exp $
 */
public class Debug {

    public static final String tag_class (String fullname) {
	int pos = fullname.indexOf ('.');
	if (pos == -1)
	    return fullname;
	return fullname.substring (0, pos);
    }

    /**
     * If debugging has been turned on for <code>tag</code>, print
     * <code>msg</code>, followed by a new line.  The tag should
     * contain the name of the file this function is called from.  
     */
    public static final void println (String msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag))))
	    _println (msg);
    }

    public static final void println (String tag, String msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag))))
	    _println (msg);
    }

    /**
     * If debugging has been turned on for <code>tag</code>, print
     * <code>tag</code>+<code>msg</code>, followed by a new line.  The tag
     * should contain the name of the file this function is called from.  
     */
    public static final void printtagln (String tag, String msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag)))) {

	    if (soss.core.EventQueue.instance != null)
		_println (soss.core.EventQueue.instance.current_time () + 
			  " " + tag + ": " + msg);
	    else
		_println (tag + ": " + msg);
	//}
    }

    private static final void _println( String msg ) {
	if( print_thread_name ) {
	    String thread_name = Thread.currentThread().getName();
	    System.err.println ("(" + thread_name + ") " + msg );
	} else {
	    System.err.println ( msg );
	}
    }

    /**
     * If debugging has been turned on for <code>tag</code>, print
     * <code>msg</code>.  The tag should contain the name of the file
     * this function is called from.  
     */
    public static final void print (String tag, String msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag))))
	    System.err.print (msg);
    }

    public static final void print (String msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag))))
	    System.err.print (msg);
    }

    public static final void print (String tag, char msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag))))
	    System.err.print (msg);
    }

    /**
     * If debugging has been turned on for <code>tag</code>, print
     * <code>tag</code>+<code>msg</code>.  The tag should contain
     * the name of the file this function is called from.  
     */
    public static final void printtag (String tag, String msg) {
	//if (_on && (_tags.containsKey (tag) 
	//            || _tags.containsKey (tag_class (tag))))
	    System.err.print (tag + ": " + msg);
    }

    /**
     * Turn on debugging for <code>tag</code>.  
     */
    public static final void turnOn (String tag) {
	if (! _on)
	    _on = true;

	_tags.put (tag, tag);
    }

    /**
     * Turn off debugging for <code>tag</code>.  
     */
    public static final void turnOff (String tag) {
	_tags.remove (tag);

	if (_tags.size () == 0)
	    _on = false;
    }

    /**
     * I use <code>_on</code> to make sure that when debugging is
     * turned off, the conditionals at the front of the print
     * functions are as fast as possible.  Also, since all the
     * functions are final, they should get inlined, so this whole
     * class shouldn't cost <i>too</i> much to use.  
     */
    private static boolean _on = true; //false;
    private static final Hashtable _tags = new Hashtable ();

    /**
     * If <code>true</code>, prepend every output with the running thread's
     * name.  We usually have a one-to-one relationship between threads and
     * stages.
     */
    private static final boolean print_thread_name = false;
}
