/*
 *  Copyright (c) 2003 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

import java.util.Enumeration;
import java.util.Vector;
import seda.sandStorm.api.ManagerIF;
import seda.sandStorm.main.stageDescr;

/**
 * Destroys all of the stages associated with some NodeID.  This implements
 * Runnable so it can be run in a new thread, if necessary.
 *
 *  @author  Jeremy Stribling
 *  @version $Id: DestroyStages.java,v 1.1 2003/02/20 23:47:54 strib Exp $
 */
public class DestroyStages implements Runnable {
    
    private NodeId node;
    private ManagerIF manager;
    
    public DestroyStages( ManagerIF manager, NodeId node ) {
	this.node = node;
	this.manager = manager;
    }

    /**
     * Goes through and destroys all stages enumerated in Manager
     * that are associated with the given NodeId.  Assumes a naming
     * convention for stages: <StageName-NodeId>.  This is the 
     * standard done by run-experiment.
     */
    public void run() {

	// determine the list of nodes
	Vector stagenames = new Vector();
	Enumeration allStages = manager.getConfig().getStages();
	while( allStages.hasMoreElements() ) {
	    stageDescr descr = (stageDescr) allStages.nextElement ();
	    int dashindex = descr.stageName.lastIndexOf( '-' );
	    if( dashindex < 0 ) {
		System.err.println( "Stage " + descr.stageName + 
				    " not in the right format." );
		continue;
	    }
	    NodeId currnode = null;
	    try {
		currnode = 
		    new NodeId( descr.stageName.substring( dashindex+1 ) );
	    } catch( Exception e ) {
		System.err.println( "DestroyStages couldn't make a NodeId out "
				    + " of " + 
				    descr.stageName.substring( dashindex+1 ) 
				    + ", exception: " + e );
		continue;
	    }
	    if( currnode.equals(node) ) {
		stagenames.add( descr.stageName );
	    }
	}

	for( int i = 0; i < stagenames.size(); i++ ) {
	    try {
		manager.destroyStage( (String) stagenames.get(i) );
	    } catch( Exception e ) {
		System.err.println( "Got exception " + e + " when trying to " +
				    " destroy stage " + 
				    (String) stagenames.get(i) );
	    }
	}
    }
    
}
