/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

/**
 *  An exception caused by incorrect or incomplete config files.
 *
 * @author	Dennis Geels
 * @version	$Id: InvalidConfigFileException.java,v 1.4 2001/10/18 00:20:03 eaton Exp $
 */
public class InvalidConfigFileException extends Exception
{
    /**
     * Constructs a new InvalidConfigFileException
     *
     * @param stage	The name of the stage throwing the exception.
     *			 May be <code>null</code>.
     * @param tag	The missing config file tag
     */
    public InvalidConfigFileException( String stage, String tag ) {
	super( "Missing or invalid tag \"" + tag + "\"" +
	       ((stage==null)?(""):(" for stage \""+stage+"\""))
	       + " in config file" );
    }

    /**
     * Constructs a new generic InvalidConfigFileException
     */
    public InvalidConfigFileException() {
	super( "Missing or invalid tag in config file" );
    }

    /**
     * Constructs a InvalidConfigFileException with an arbitrary message.
     * This constructor simply calls the constructor for
     * {@link Exception Exception}.  It should be called by subclasses of
     * <code>InvalidCOnfigFileException</code> which do not want to use
     * this class' error message.
     *
     * @param msg	The message for this <code>Exception</code>.
     */
    public InvalidConfigFileException( String msg ) {
	super( msg );
    }

}
