/**
 * NonceAckMsg
 *
 * An acknowledgement for NonceMsgs.
 *
 * @author     Jeremy Stribling
 * @version $Id: NonceAckMsg.java,v 1.1 2003/03/03 19:34:50 strib Exp $ */

package ostore.util;
import java.util.Vector;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.QSException;

public class NonceAckMsg 
extends NetworkMessage implements QuickSerializable {

    private Vector nonces;
    public short stage_id;
    private int[] nonces_ar;

    public NonceAckMsg ( NodeId peer ) {
      super(peer, false /*outbound*/);
      nonces = new Vector(1);
      nonces_ar = null;
    }

    public NonceAckMsg ( InputBuffer buffer ) throws QSException {
      super (buffer); 
      stage_id = buffer.nextShort();
      int size = buffer.nextInt();
      nonces = new Vector( size );
      nonces_ar = new int[ size ];
      for( int i = 0; i < size; i++ ) {
	  int nonce = buffer.nextInt();
	  nonces.add( new Integer(nonce) );
	  nonces_ar[i] = nonce;
      }
    }

    public void serialize ( OutputBuffer buffer ) {
      super.serialize (buffer);
      buffer.add( stage_id );
      buffer.add( nonces.size() );
      for( int i = 0; i < nonces.size(); i++ ) {
	  buffer.add( ((Integer)nonces.get(i)).intValue() );
      }
    }
    
    public void setStageId( short stageId ) {
	this.stage_id = stageId;
    }

    public short getStageId() {
	return stage_id;
    }
    
    public void addNonceAck( int nonce ) {
	nonces.add( new Integer( nonce ) );
	// invalidate cached array
	nonces_ar = null;
    }

    public int[] getNonceAcks() {
	if( nonces_ar == null ) {
	    nonces_ar = new int[ nonces.size() ];
	    for( int i = 0; i < nonces_ar.length; i++ ) {
		nonces_ar[i] = ((Integer)nonces.get(i)).intValue();
	    }
	}
	return nonces_ar;
    }

    public Object clone () throws CloneNotSupportedException {
	NonceAckMsg newone = (NonceAckMsg) super.clone ();
	for( int i = 0; i < nonces.size(); i++ ) {
	    newone.addNonceAck( ((Integer) nonces.get(i)).intValue() );
	}
	newone.setStageId( stage_id );
	return newone;
    }

}

