/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

/**
 * A dull mirror of the <code>java.lang.String</code> class which implements
 * <code>ostore.util.QuickSerializable</code>.  Most of the functionality from
 * the <code>String</class> has not been ported; cast the
 * <code>QSString</code> using <code>toString()</code> and call the
 * <code>String</code> methods directly.
 *
 *  @author  Dennis Geels
 *  @version $Id: QSString.java,v 1.14 2002/07/29 18:48:54 geels Exp $
 */
public class QSString implements QuickSerializable, Comparable
{
    //////////////////// 	CONSTANTS	////////////////////
    //////////////////// 	VARIABLES	////////////////////

    private String _value;

    ////////////////////    STATIC VARIABLES 	////////////////////
    //////////////////// 	 METHODS	////////////////////

    public void serialize(OutputBuffer buffer)
    {
        buffer.add(_value);
        return;
    }

    //  methods for ostore.util.QuickSerializable interface
    //  end methods for ostore.util.QuickSerializable interface


    public int length()
    {
	return _value.length();
    }

    public boolean equals(Object other)
    {
	boolean matched;
	if( other == this ) {
	    matched = true;
	} else if (! (other instanceof QSString)) {
            matched = false;
        } else {
	    matched = _value.equals(((QSString)other).toString());
	}
        return matched;
    }

    public int hashCode()
    {
        return _value.hashCode();
    }

    public int compareTo(Object o)
    {
        return _value.compareTo(o);
    }

    public String toString()
    {
        return _value;
    }

    ////////////////////     STATIC METHODS 	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////

    public QSString()
    {
	_value = "";
        return;
    }

    public QSString(String s)
    {
        _value = s;
        return;
    }

    public QSString(StringBuffer sb)
    {
        _value = new String(sb);
        return;
    }

    public QSString(InputBuffer buffer)
        throws ostore.util.QSException
    {
        _value = buffer.nextString();
        return;
    }
}
