/**
 * RandomAccessFileInputBuffer.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.util;
import java.io.RandomAccessFile;
import java.io.IOException;

/**
 * For use in testing only; not efficient at all, and not completely
 * implemented either.  See ostore.mac.HMACDHHandler for an example use.
 *
 * @author	Sean C. Rhea
 * @version	$Id: RandomAccessFileInputBuffer.java,v 1.1 2002/06/30 04:20:37 srhea Exp $
 */
public class RandomAccessFileInputBuffer extends InputBufferImpl {
    
    protected RandomAccessFile file;

    public RandomAccessFileInputBuffer (RandomAccessFile file) {
	this.file = file;
    }

    public byte nextByte () {
	try {
	    return (byte) (0xff & file.read ());
	}
	catch (IOException e) {
	    throw new AssertionViolatedException ("IOException: " + e);
	}
    }

    public void nextBytes (byte [] data, int offset, int length) {
	try {
	    file.read (data, offset, length);
	}
	catch (IOException e) {
	    throw new AssertionViolatedException ("IOException: " + e);
	}
    }

    public void nextBytes (OutputBuffer outb) {
	throw new NoSuchMethodError (); // not supported
    }

    public InputBuffer subBuffer (int length) {
	throw new NoSuchMethodError (); // not supported
    }

}

