/**
 * SHA1NativeProvider.java
 *
 * @author   Hakim Weatherspoon
 * @version  $Id: SHA1NativeProvider.java,v 1.1 2002/08/28 03:20:19 hweather Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 **/
package ostore.util;

import java.security.Provider;
import java.security.AccessController;

/**
 * <CODE>SHA1HashNativeProvider</CODE> is the 
 * {@link java.security.Provider} for the <CODE>SHA-1</CODE>
 * <CODE>native</CODE> implementation in <CODE>C</CODE>.

 * The only algorithm supported, and their names is the 
 * <CODE>SHA</CODE>; the message digest scheme described in FIPS 180-1. 
 * Aliases for SHA are SHA-1 and SHA1.
 *
 * @author   Hakim Weatherspoon
 * @version  $Id: SHA1NativeProvider.java,v 1.1 2002/08/28 03:20:19 hweather Exp $
 **/
public final class SHA1NativeProvider extends Provider {

  ////////////////////       CONSTANTS		////////////////////
  ////////////////////    STATIC VARIABLES 	////////////////////

  private static final String INFO = "SHA1Native " + 
    "(SHA-1 digest)";

  ////////////////////       VARIABLES		////////////////////
  ////////////////////  STATIC CONSTRUCTORS	////////////////////
  ////////////////////      CONSTRUCTORS	//////////////////// 

  /**
   * Construct according to the given fields. */
  public SHA1NativeProvider() 
  {
    /* We are the SHA1Native provider */
    super("SHA1Native", 1.0, INFO);

    //AccessController.doPrivileged(new java.security.PrivilegedAction() 
    //{
    //  public Object run() 
    //  {
          /* 
	   * Digest engines */
          put("MessageDigest.SHA", "ostore.util.SHA1NativeIF");
	
	  put("Alg.Alias.MessageDigest.SHA-1", "SHA");
	  put("Alg.Alias.MessageDigest.SHA1", "SHA");

	  /*
	   * Implementation type: software or hardware */
	  put("MessageDigest.SHA ImplementedIn", "Software");
	  
    //    return null;
    //  }
    //});
  }

  ////////////////////     STATIC METHODS 	////////////////////
  ////////////////////	 	METHODS		////////////////////
  ////////////////////     INNER CLASSES	////////////////////
}
