# Toolbox.pm
#
# $Id: Toolbox.pm,v 1.53 2003/03/28 00:58:36 strib Exp $
#
# Useful common subroutines and data structures for regression test scripts.

package Toolbox;

#use strict;

BEGIN {
    # Assume Toolbox found in pond/regression:
    my $regression_dir = $ENV{"PWD"} . "/" . $INC{"Toolbox.pm"};
    $regression_dir =~ m%(.*)/regression%;
    $ENV{"OSTORE_RUN_DIR"} = "$1/run";
    print "RUN: $ENV{OSTORE_RUN_DIR}\n";
}
use lib $ENV{"OSTORE_RUN_DIR"};
use Tools;

use Exporter();
@ISA = (Exporter);
@EXPORT = qw( $ostore_home $script_name
	      $FED_INDEX $RP_INDEX $CLIENT_INDEX 
	      &fork_simulator &rewrite
	      &done &croak);

# Add subroutines defined in ../run/Tools.pm.
push @EXPORT, qw( &wait_for &wait_for_ready &wait_for_tapestry_ready
		  &wait_for_network_ready &returned_zero &is_clean
		  &override &insert &interpolate &remote_run_dir &is_localhost
		  &fork_process &fork_sandstorm &fork_java &die
		  %ostore_dirs $seed_counter $disable_jit
		  $tmp_dir $hostname @child_pids @logs @configs @dirs @profiles );
		  
$| = 1;

# Script name - reasonable default
$script_name = $0;
$script_name =~ s|.*/||;

# TODO: don't use default name ever - add specialized name to @profiles
push @profiles, "/tmp/sandstorm-graph.txt";	# a common one

# Define more directories.
$ostore_home = $ostore_dirs{"run_dir"} . "/../ostore";
my $key_dir = $ostore_dirs{"run_dir"} . "/keys";


# Determines which set of keyshares the inner ring uses.
#  Default: 512-bit keys (ir.pub, ir-0.priv, etc.)
my $keyshare_prefix = ($ENV{"KEYSHARE_PREFIX"} or "ir");


# fork a process to run soss; return its PID.
# redirect STDOUT and STDERR to the named log.
# automatically adds the PID to @child_pids and the log filename to @logs.
#
# Usage: &fork_simulator( <log_filename>, <config_file> );

sub fork_simulator {
    my ($log, $cfg) = @_;

    &fork_java( $log, "soss.core.Main", $cfg );
}

########################################
# Common value map, with index-independent values
# We need to triple-backslash the keys so that perl doesn't dereference them.

my %value_map;
my $fed_port = 3630;
my $rp_port = $fed_port+2;
my $client_port = $fed_port+4;
my @pr_pkeys = glob( "$key_dir/pr-*pub" );
my @pr_skeys = glob( "$key_dir/pr-*priv" );
my @ir_keyshares = glob( "$key_dir/$keyshare_prefix-?.priv" );
my @storage_pkeys = glob( "$key_dir/storage-*pub" );
my @storage_skeys = glob( "$key_dir/storage-*priv" );

$value_map{"Host"} = "$hostname";

$value_map{"FederationID"} = "$hostname:$fed_port";
$value_map{"FederationPort"} = "$fed_port";

$value_map{"MemberPublicKeys"} = "@pr_pkeys";
$value_map{"BasePort"} = $fed_port + 6;
$value_map{"GatewayID"} = "$hostname:$value_map{BasePort}";

my $i;
for( $i=0; $i<scalar(@pr_pkeys); $i++ ){
    $value_map{"MemberPublicKey\\\[$i\\\]"} = "$pr_pkeys[$i]";
    $value_map{"MemberPrivateKey\\\[$i\\\]"} = "$pr_skeys[$i]";
    my $port = $value_map{"BasePort"} + $i*2;    
    $value_map{"MemberID\\\[$i\\\]"} = "$hostname:$port";
}

$value_map{"IRPublicKey"} = "$key_dir/$keyshare_prefix.pub";
$value_map{"IRPrivateKeyShares"} = "@ir_keyshares";
for( $i=0; $i<scalar(@ir_keyshares); $i++ ){
    $value_map{"IRPrivateKeyShare\\\[$i\\\]"} = "$ir_keyshares[$i]";
}

for( $i=0; $i<scalar(@storage_pkeys); $i++ ){
    $value_map{"StoragePublicKey\\\[$i\\\]"} = "$storage_pkeys[$i]";
    $value_map{"StoragePrivateKey\\\[$i\\\]"} = "$storage_skeys[$i]";
    my $port = $value_map{"BasePort"} + $i*2;    
    $value_map{"MemberID\\\[$i\\\]"} = "$hostname:$port";
}

$value_map{"RPID"} = "$hostname:$rp_port";
$value_map{"RPPort"} = "$rp_port";
$value_map{"RPPublicKey"} = "$key_dir/rp.pub";
$value_map{"RPPrivateKey"} = "$key_dir/rp.priv";

$value_map{"PrincipalPublicKey"} = "$key_dir/principal.pub";
$value_map{"PrincipalPrivateKey"} = "$key_dir/principal.priv";

# TODO: client could be generic
$value_map{"ClientID"} = "$hostname:$client_port";
$value_map{"ClientPort"} = "$client_port";
$value_map{"ClientPublicKey"} = "$key_dir/client.pub";
$value_map{"ClientPrivateKey"} = "$key_dir/client.priv";
    
$FED_INDEX = -1;
$RP_INDEX = -2;
$CLIENT_INDEX = -3;

my %tags;
$tags{$FED_INDEX} = "federation";
$tags{$RP_INDEX} = "rp";
$tags{$CLIENT_INDEX} = "client";

# Produces sandstorm configuration files with runtime values.
#
# The "index" parameter should be the index of the configured machine, if
#   it is a member of the inner ring.  Otherwise use one of the "*_INDEX"
#   constants above.
# Usage: &rewrite( <template>, <output_filename>, <index> )

sub rewrite {
    my ($template, $output, $index) = @_;

    # fill in index-specific portions of %value_map.
    my $port = $value_map{"BasePort"} + $index*2;
    my $tag = $tags{$index};
    if( ! defined $tag ) {
	$tag = "node-$index";
    }
    $value_map{"OSTOREHOME"} = "$ostore_home";
    $value_map{"MemberIndex"} = "$index";
    $value_map{"MemberID"} = "$hostname:$port";
    $value_map{"MemberPort"} = "$port";
    $value_map{"MemberPublicKey"} = "$key_dir/pr-$index.pub";
    $value_map{"MemberPrivateKey"} = "$key_dir/pr-$index.priv";
    $value_map{"StoragePublicKey"} = "$key_dir/storage-$index.pub";
    $value_map{"StoragePrivateKey"} = "$key_dir/storage-$index.priv";
    $value_map{"IRPrivateKeyShare"} = "$key_dir/ir-$index.priv";
    $value_map{"SSProfileFile"} = "$tmp_dir/$script_name-profile-$tag-$$.txt";
    $value_map{"SSGraphFile"} = "$tmp_dir/$script_name-graph-$tag-$$.txt";
    $value_map{"CacheDir"} = "$tmp_dir/$script_name-blocks-$tag-$$/";

    # Add the newly created filenames to appropriate lists.
    push @configs, $output;
    push @dirs, $value_map{"CacheDir"};
    push @profiles, $value_map{"SSProfileFile"};
    push @profiles, $value_map{"SSGraphFile"};
    
    # Now write the new, interpolated file.
    &interpolate( $template, $output, \%value_map );
}


# Clean up.
#
# Usage: &done( <exit_code> )
sub done {
    my $error_code = shift;
    
    if ($error_code == 0) {
	print "*** $script_name: Success! ***\n";
    } else {
	print "*** $script_name: Failure! Logs follow ***\n";
	foreach $log ( @logs ) {
	    if( -f $log ) {
		system( "cat", $log );
	    }
	}
    }

    &clean_up( $error_code );
    
    print "*** $script_name: Goodbye ***\n";
    exit $error_code;
}

# Handle signals
sub croak {
    &done( 1 );
}

# Set up a signal handler to clean up if the script is killed.
$SIG{INT} = 'croak';
$SIG{QUIT} = 'croak';
$SIG{HUP} = 'croak';
$SIG{PIPE} = 'croak';
$SIG{TERM} = 'croak';
