#!/usr/bin/perl -w
#
# author: Jeremy Stribling
# $Id: get_current_plservers.pl,v 1.4 2003/02/14 21:08:38 strib Exp $
#
# gets the current planetlab servers, and if any were 
# different from the last list, 
# copy relevant code onto the new nodes

# first, ask for the entire list of planet lab nodes
my $tmp_file = "/tmp/get_current_plservers.$$.tmp";
my $curr_file = "current_plservers";
my $gray_file = "current_graylist";
my $SECOND_CHANCES = 4;
if( ! system( "./fetch-planetlab-list.pl --nocheckapp " . 
	      "&> $tmp_file" ) ) {

    # now, check whether or not there are new nodes in this list
    open( TMP, "<$tmp_file" ) or die ( "Couldn't open $tmp_file" );
    my @tmp = <TMP>;
    close( TMP );
    unlink( $tmp_file );

    # should be in the last line of the file
    my @servers = &get_server_array( $tmp[$#tmp] );

    # open up the current list of nodes
    open( CURR, "<$curr_file" ) or die( "Couldn't open $curr_file" );
    my @curr = <CURR>;
    close( CURR );
    
    # should be the only line
    if( $#curr < 0 ) {
	die( "Wrong format for $curr_file" );
    }
    my @curr_servers = &get_server_array( $curr[0] );

    # now compare
    my $changed = 0;
    foreach $server (@servers) {
	if( ! grep( /$server/, @curr_servers ) ) {
	    # if it's not in there already, try to upload code onto it
	    print "uploading to $server\n";
	    my $rt_val = system( "export OSTORE_SERVERS=$server; " .
				 "./fetch_host_keys; " );
	    if( !$rt_val ) {
		# breaking these calls up will speed things up in 
		# case of timeouts
		$rt_val = system( "export OSTORE_SERVERS=$server; " .
				  "./call_remote.pl mkdir -p " . 
				      "strib/oceanstore/run; " .
				  "./copy_remote.pl --remote-dir " . 
				      "strib/oceanstore/run Tools.pm " . 
				      "Traceroute.pm ping_times.pl" );
	    }
	    # if it worked, add it to the list of servers
	    if( !$rt_val ) {
		push @curr_servers, $server;
		$changed = 1;
	    } else {
		print "upload to $server failed\n";
	    }
	}
    }

    # open up the current list of nodes
    open( GRAY, "<$gray_file" ) or die( "Couldn't open $gray_file" );
    my @gray = <GRAY>;
    close( GRAY );

    # now if a node is really bad, it might disappear from the
    # ./fetch-planetlab-hosts list, so give it a few chances before
    # taking it out of curr_servers
    my @new_grays = ();
    foreach $server (@curr_servers) {
	if( ! grep( /$server/, @servers ) ) {
	    # infospect thinks this one is bad
	    push @new_grays, $server;
	}
    }

    # go through the new grays and update
    # notice this will only keep those in the gray list that don't appear
    # in the fetch-planetlab list
    my @grays_out = ();
    foreach $server (@new_grays) {
	my @trues = grep( /$server/, @gray );
	if( !@trues ) {
	    push @grays_out, "$server 1"; 
	} else {	    
	    # assume it doesn't appear more than once
	    my @splitline = split( /\s/, $trues[0] );
	    if( $#splitline != 1 ) {
		print "Badly formatted line in $gray_file: $trues[0]\n";
		next;
	    }
	    my $new_val = $splitline[1]+1;
	    if( $new_val > $SECOND_CHANCES ) {
		# now if its been grayed out too many times, remove it from 
		# curr_servers
		my $index = 0;
		foreach $cs (@curr_servers) {
		    if( $cs eq $server ) {
			splice( @curr_servers, $index, 1 );
			$changed = 1;
			last;
		    } else {
			$index++;
		    }
		}
	    } else {
		push @grays_out, "$server $new_val";
	    }
	}
    }

    # print out the grays
    open( GRAY, ">$gray_file" ) or die( "couldn't open $gray_file " . 
					"for writing" );
    foreach $line (@grays_out) {
	print GRAY "$line\n";
    }
    close( GRAY );

    # if anything changed, write out a new file
    if( $changed ) {
	open( CURR, ">$curr_file" ) or die( "couldn't open $curr_file " . 
					    "for writing" );
	foreach $server (@curr_servers) {
	    print CURR "$server ";
	}
	print CURR "\n";
	close( CURR );
    }

}

sub get_server_array {

    my $export_line = shift;

    my @eq_split = split( /=/, $export_line );
    if( $#eq_split == -1 or $#eq_split > 1 ) {
	die( "get_server_array line not in right format: $export_line " );
    }
    #strip off the quotes
    $eq_split[$#eq_split] =~ s/\"//g;
#    print "$eq_split[1]\n";
    my @servers = split( /\s/, $eq_split[$#eq_split] );

    return @servers;

}
