/**
 * DisseminateAnalyzer.java
 *
 * Copyright (c) 2002-2003 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.PacketAnalyzers;

import Sonar.*;
import Sonar.util.*;
import Sonar.event.*;
import Sonar.Dialog.*;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.zip.*;

import ostore.util.NodeId;
import ostore.util.QuickSerializable;
import ostore.tapestry.impl.RouteMsg;
import ostore.tapestry.impl.RouteObjRootMsg;
import ostore.tapestry.impl.RouteObjDestMsg;

import ostore.apps.visdemo.VisUp;
import ostore.network.NetworkMessage;

import ostore.archive.dissemination.DisseminateFragMsg;
import ostore.rp.CreateObjectReqMsg;
import ostore.rp.CreateObjectReq;
import ostore.rp.InnerRingCreateReqMsg;
import ostore.rp.ObjectCreateReqMsg;
import ostore.agree.ForwardMsg;
import ostore.agree.ResultMsg;
import ostore.archive.RequestFragMsg;

import ostore.inner.UpdateReqMsg;

//SANDSTORM
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.NodeId;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;

import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.internal.SinkProxy;

/**
 * Merges all the previously separate Analyzers for test-run.archive.diss
 * test.
 * 
 * @author   Jason Lee <jlee81@cs.berkeley.edu>
 */

public class DisseminateAnalyzer extends PacketAnalyzer implements EventHandlerIF, ProfilableIF, SingleThreadedEventHandlerIF {

    protected TwoKeyHashtable proprietary_edge_info;
    protected long edgeEventLongevity = 2 * 1000;
    protected boolean _show_along_tapestry = false;
    
    public void inst () {
	proprietary_edge_info = new TwoKeyHashtable();
    }

    protected void checkEdges() {
	final String tag = class_tag + ".checkEdges";
	EdgeInfo tempEdge = null;
	long currentTime = Calendar.getInstance().getTime().getTime();
	try {
	    for(Enumeration edges = proprietary_edge_info.elements(); 
		    edges.hasMoreElements();) {
		tempEdge = (EdgeInfo)edges.nextElement(); 
		if (currentTime - edgeEventLongevity > tempEdge.getTime() 
			&& currentTime - edgeEventLongevity > tempEdge.getTime()
			&& tempEdge.getInstances() == -1 ) {
		    proprietary_edge_info.remove(tempEdge.getSource(),
			    tempEdge.getDest());
		    
		    /*if(DEBUG) Debug.printtagln(tag, "resetting the source, dest nodes");
		    _classifier.dispatch(new NodeEvent(this, tempEdge.getSource(), 
				Calendar.getInstance().getTime(), NodeEvent.ID,
				DisplayManager.NEVER_SET));
		    _classifier.dispatch(new NodeEvent(this, tempEdge.getDest(),
				Calendar.getInstance().getTime(), NodeEvent.ID,
				DisplayManager.NEVER_SET));
				*/
		    
		    
		} else if(currentTime - edgeEventLongevity > tempEdge.getTime() 
			&& tempEdge.getInstances() >= 0) {
		    if(tempEdge.getInstances() > 1) {
			tempEdge.instances--;
			_classifier.dispatch(new EdgeEvent(this, tempEdge,
				    Calendar.getInstance().getTime(), 
				    EdgeEvent.MODIFY_WIDTH));
		    } else {
			_classifier.dispatch(new EdgeEvent(this, tempEdge,
				    Calendar.getInstance().getTime(), EdgeEvent.DELETE));
			tempEdge.instances = -1;
		    }	
		}
	    }
	} catch (SinkFullException e) {
	    e. printStackTrace();
	}

	Signal.getSignal(_self_node_id).register(Signal.ALRM, stageName, _this_sink);
	Signal.getSignal(_self_node_id).alarm(stageName, ALARM_SLEEP);
    }
    
    public void handleMsg (VisUp msg) {
	final String tag = class_tag + ".handleMsg";
	int offset[] = {0};
	int typeData = -1;

	NodeId sender = null; 
	NodeId recp  = null; 
	
	NodeId rp = null;
	NodeId client = null;
	NodeId storageServer = null;
	NodeId innerRing = null;
	NodeId set_creator = null;
	NodeId replica = null;

	String msg_type = "";

	/*
	if(e.hasPayload()) {
	    RouteMsg rt_msg = null;
	    RouteObjRootMsg ror_msg = null;
	    RouteObjDestMsg rod_msg = null;

	    if(e.getPacket() instanceof RouteMsg) {
		rt_msg = (RouteMsg) e.getPacket(); 
	    } else if(e.getPacket() instanceof RouteObjRootMsg) {
		ror_msg = (RouteObjRootMsg) e.getPacket();
	    } else if(e.getPacket() instanceof RouteObjDestMsg) {

	    }
	    try { 
		if(rt_msg != null && rt_msg.user_data() != null) {
		    MainClass.outv("DA> Received a message for dissemination test");
		    if(rt_msg.user_data() instanceof CreateObjectReqMsg) {
			MainClass.outv("DA> create object req msg received");
			CreateObjectReqMsg msg = (CreateObjectReqMsg) rt_msg.user_data();
			client = MainClass.objectMaintainer.guid_to_node(msg.req().client);
			rp = MainClass.objectMaintainer.guid_to_node(rt_msg.dest);
			sender = client;
			recp = rp;
			msg_type = "ostore.rp.CreateObjectReqMsg";
		    } else if(rt_msg.user_data() instanceof DisseminateFragMsg)  {
			MainClass.outv("DA> diss frag msg received");
			innerRing = MainClass.objectMaintainer.guid_to_node(rt_msg.src);
			storageServer = MainClass.objectMaintainer.guid_to_node(rt_msg.dest);

			sender = innerRing;
			recp = storageServer;

			msg_type = "ostore.archive.dissemination.DisseminateFragMsg";
		    } else if(rt_msg.user_data() instanceof InnerRingCreateReqMsg ||
			    (MainClass.analyzer_menu.show_ir_traffic && 
			     (rt_msg.user_data() instanceof ObjectCreateReqMsg ||
			      rt_msg.user_data() instanceof ForwardMsg ||
			      rt_msg.user_data() instanceof ResultMsg))) {
			MainClass.outv("DA> recieved an inner ring msg:"+rt_msg.user_data().getClass().getName());
			innerRing = MainClass.objectMaintainer.guid_to_node(rt_msg.dest);
			sender = MainClass.objectMaintainer.guid_to_node(rt_msg.src);
			recp = innerRing;

			msg_type = "ostore.rp.InnerRingCreateReqMsg";

		    } else {
			return;
		    }
		} else if(ror_msg != null && rod_msg != null) {
		    QuickSerializable msg_data = null;
		    if(ror_msg != null && ror_msg.user_data() != null) {
			msg_data = ror_msg.user_data();
		    } else if(rod_msg != null && rod_msg.user_data() != null) {
			msg_data = rod_msg.user_data();

			sender =  MainClass.objectMaintainer.guid_to_node(rod_msg.src);
			recp = MainClass.objectMaintainer.guid_to_node(rod_msg.dest);
			//MainClass.outv("URA> src:"+rt_msg.src+" guid:"+rt_msg.guid);
		    }

		    if(msg_data instanceof UpdateReqMsg) {
			UpdateReqMsg locate_msg = (UpdateReqMsg) msg_data;
			msg_type = "ostore.inner.UpdateReqMsg";
		    } else if (msg_data instanceof RequestFragMsg) {
			msg_type = "ostore.archive.RequestFragMsg";
		    } else 
			return;
		} else 
		    return; //not handled type
	    } catch (Exception qse) {
		qse.printStackTrace();
	    }
	} else { 
	*/    
	
	if(msg.hasPayload()) return;
	    
	//If only sending strings containing the queried msg type	
	if(_hash_to_node_map == null || _node_to_hash_map == null) return;
	sender = (NodeId) _hash_to_node_map.get(msg.getSource());
	recp   = (NodeId) _hash_to_node_map.get(msg.getDest());

	if(msg.getMsgName().equals("ostore.rp.CreateObjectReqMsg")) {
	    client = sender;
	    rp = recp;
	    msg_type = msg.getMsgName();
	    if(DEBUG) Debug.printtagln(tag, "received a CreateObjectReqMsg");
	} else if (msg.getMsgName().equals("ostore.archive.dissemination.DisseminateFragMsg")) {
	    innerRing = sender;
	    storageServer = recp;
	    msg_type = msg.getMsgName();
	} else if(msg.getMsgName().equals("ostore.rp.InnerRingCreateReqMsg") ||
		(/*MainClass.analyzer_menu.show_ir_traffic && */
		 (msg.getMsgName().equals("ostore.rp.ObjectCreateReqMsg") ||
		  msg.getMsgName().equals("ostore.rp.CreateObjectReq") ||
		  msg.getMsgName().equals("ostore.agree.ForwardMsg") ||
		  msg.getMsgName().equals("ostore.agree.ResultMsg"))) ) {
	    innerRing = recp;
	    msg_type = "ostore.rp.InnerRingCreateReqMsg"; //e.getMsgName();
	} else if(msg.getMsgName().equals("ostore.inner.UpdateReqMsg")) {
	    msg_type = "ostore.rp.UpdateReqMsg";
	} else if(msg.getMsgName().equals("ostore.archive.RequestFragResponseMsg")) {
	    msg_type = "ostore.archive.RequestFragResponseMsg";

	    //Heartbeat/Planetlab
	} else if(msg.getMsgName().equals("ostore.archive.repair.DataDrivenServerHbMsg")) {
	    msg_type = "ostore.archive.repair.DataDrivenServerHbMsg";
	} else if(msg.getMsgName().equals("ostore.tapestry.patchwork.PatchworkFaultMsg")) {
	    msg_type = "ostore.tapestry.patchwork.PatchworkFaultMsg";
	} else if(msg.getMsgName().equals("ostore.apps.mail.core.test.StartPrefetch")) {
	    msg_type = "ostore.apps.mail.core.test.StartPrefetch";
	    replica = recp;
	} else {
	    return;
	}

	//if(DEBUG) Debug.printtagln(tag, "received a "+msg_type);

	//Hop to hop (then the sender/recp is overwritten)
	//TODO create a new event to alert the packet analyzers (from analyzer menu)
	if(_show_along_tapestry) {
	    sender = msg.getSender();
	    recp  = msg.getRecp(); 
	}
	

	//Trigger an event for the identification of nodes
	try {
	    if(client != null) {
		_classifier.dispatch(new NodeEvent(this, client, Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.CLIENT));
	    }
	    if(storageServer != null) {
		_classifier.dispatch(new NodeEvent(this, storageServer, Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.FILE_SERVER));
	    }
	    if(innerRing != null) {
		_classifier.dispatch(new NodeEvent(this, innerRing, Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.INNER_RING));
	    }
	    if(rp != null) {
		_classifier.dispatch(new NodeEvent(this, rp, Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.RESP_PARTY));
	    }
	    if(replica != null) {
		_classifier.dispatch(new NodeEvent(this, replica, Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.REPLICA));
	    }

	    //New Edge Event
	    if(sender != null && recp != null) {
		EdgeInfo edgeInfo = null;
		if((edgeInfo = (EdgeInfo) proprietary_edge_info.get(sender, recp)) != null) {
		    edgeInfo.instances++;
		    edgeInfo.modifyTime(Calendar.getInstance().getTime().getTime());
		    _classifier.dispatch(new EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime(), EdgeEvent.MODIFY_WIDTH));
		} else {
		    edgeInfo = new EdgeInfo(sender, recp, msg_type, 
			    Calendar.getInstance().getTime().getTime());
		    proprietary_edge_info.put(sender, recp, edgeInfo); 
		    _classifier.dispatch(new EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime(), EdgeEvent.CREATE));
		}
	    } 
	} catch (SinkFullException e) {
	    e.printStackTrace();
	}


    }

    /**
     * SANDSTORM 
     */
    private static final String class_tag = "DisseminateAnalyzer";
    private static final boolean DEBUG = false;

    protected Map _node_to_hash_map = null;
    protected Map _hash_to_node_map = null;

    public void handleEvent (QueueElementIF item) throws EventHandlerException {
	final String tag = class_tag + ".handleEvent";
	if (item instanceof VisUp) {
	    handleMsg ((VisUp) item);
	} else if (item instanceof Signal.AlarmSignal) {
	    checkEdges();
	} else if (item instanceof GuidMapEvent) {
	    _node_to_hash_map = ((GuidMapEvent) item).getNodeToHash();
	    _hash_to_node_map = ((GuidMapEvent) item).getHashToNode();

	}
    }

    public void handleEvents (QueueElementIF items[]) throws EventHandlerException {
	final String tag = class_tag + ".handleEvents";
	for(int i=0; i<items.length; i++ ) 
	    handleEvent(items[i]);
    }

    public void init(ConfigDataIF config) throws Exception {
	final String tag = class_tag + ".init";

	//Sets all the variables to initial values
	inst();

	Filter filter = null;
	String messageTypes[] = {"ostore.apps.visdemo.VisUp",
				 "Sonar.event.GuidMapEvent"};
	
	if (DEBUG) Debug.turnOn(class_tag);
	if (DEBUG) Debug.printtagln (tag, "Debug turned on ");

	_self_node_id = new NodeId(config.getString("node_id"));
	_classifier = Classifier.getClassifier ( _self_node_id);

	stageName = config.getStage().getName();
	if(config.getBoolean("Profile")) {
	    config.getManager().getProfiler().add("Sonar.PacketAnalyzers.DisseminateAnalyzer", this);
	    SinkIF sink = config.getStage().getSink();
	    _this_sink = new SinkProxy(sink, config.getManager(), 
		    config.getStage().getWrapper());
	} else 
	    _this_sink = config.getStage().getSink();

	for(int i = 0; i < messageTypes.length; i++) {
	    filter = new Filter();
	    if(!filter.requireType(Class.forName (messageTypes[i])))
		BUG ( tag + ": could not require type==" +
			messageTypes[i] + ".");
	    _classifier.subscribe (filter, _this_sink);
	}

	Signal.getSignal(_self_node_id).register(Signal.ALRM, stageName, _this_sink);
	Signal.getSignal(_self_node_id).alarm(stageName, ALARM_SLEEP);
    }
    private final long ALARM_SLEEP = 150;

    // methods for ProfilableIF
    public int profileSize()
    {
	return _this_sink.size();
    }
    // end methods for ProfilableIF

    // methods for EventHandlerIF
    public void destroy () throws Exception { }
    // end methods for EventHandlerIF

    private static final void BUG (String msg) {
	Carp.die (msg + " This is a bug in " + class_tag + ". " +
		"Please let Jason know about it ASAP." );
    }

    private NodeId _self_node_id;
    private String stageName;
    private Classifier _classifier;
    private SinkIF _this_sink;
}
