/**
 * HeartBeatAnalyzer.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.PacketAnalyzers;

import Sonar.*;
import Sonar.util.*;
import Sonar.event.*;
import Sonar.Dialog.*;
import Sonar.Packet.*;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.zip.*;

import ostore.util.NodeId;
import ostore.tapestry.impl.PublishMsg;
/**
 * @author	Jason Lee <jlee81@cs.berkeley.edu>
 */

public class HeartBeatAnalyzer extends PacketAnalyzer implements Runnable {
    protected TwoKeyHashtable proprietary_edge_info;
    protected Thread del_edge_thread;
    
    public HeartBeatAnalyzer() {
	proprietary_edge_info = new TwoKeyHashtable();
	del_edge_thread = new Thread(this);
	del_edge_thread.start();
    }

    public void run() {
	EdgeInfo tempEdge = null;
	while (del_edge_thread == Thread.currentThread()) {
	    long currentTime = Calendar.getInstance().getTime().getTime();
	    for(Enumeration edges = proprietary_edge_info.elements();
		    edges.hasMoreElements();) {
		tempEdge = (EdgeInfo)edges.nextElement(); 
		if(currentTime - MainClass.edgeEventLongevity >
			tempEdge.getTime()) {
		    if(tempEdge.getInstances() > 1) {
		        tempEdge.instances--;
			MainClass.displayManager.modifyEdgeWidth(tempEdge);
		    } else {
			proprietary_edge_info.remove(tempEdge.getSource(), 
				tempEdge.getDest());
		    	MainClass.objectMaintainer.triggerEdgeDeletedEvent(
				new EdgeEvent(this, tempEdge, Calendar.getInstance().getTime()));
		    }	
		}
	    }
	}
    }
    
    public void packetReceived(PacketEvent e) {
	/*
	 * tapestry publish messages
	 * tag - what type of obj
	 * FragmentTag
	 */

	NodeId sender = null; 
	NodeId recp  = null; 
	boolean hasTag = false;

	if(!e.hasPayload() || !(e.getPacket() instanceof PublishMsg)) {
	    return;
	}

	PublishMsg msg = (PublishMsg) e.getPacket();
	if(msg.tag instanceof ostore.archive.FragmentTag) {
	    hasTag = true;
	} 
	try {
	    MainClass.outv("HeartBeat> received a "+msg.getClass().getName());

	} catch (Exception qse) { qse.printStackTrace(); }


	if(MainClass.analyzer_menu.show_along_tapestry) {
	    sender =  e.getSender();
	    recp  =  e.getRecipient(); 
	}


	if(sender != null && recp != null && hasTag) { 
	    EdgeInfo edgeInfo = null;
	    if((edgeInfo = (EdgeInfo) proprietary_edge_info.get(sender, recp )) != null) {
		edgeInfo.instances++;
		edgeInfo.modifyTime(Calendar.getInstance().getTime().getTime());
		MainClass.displayManager.modifyEdgeWidth(edgeInfo);

	    } else {
		edgeInfo = new EdgeInfo(sender, recp, "ostore.tapestry.impl.PublishMsg", Calendar.getInstance().getTime().getTime());
		edgeInfo.setOrigin(MainClass.objectMaintainer.guid_to_node(msg.src));
		proprietary_edge_info.put(sender, recp, edgeInfo); 
		MainClass.objectMaintainer.triggerEdgeCreatedEvent(new
			EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime()));
	    }
	}
    }
}
