/**
 * ICMPAnalyzer.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.PacketAnalyzers;

import Sonar.*;
import Sonar.util.*;
import Sonar.event.*;
import Sonar.Dialog.*;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.zip.*;

import ostore.tapestry.impl.RouteMsg;
import ostore.tapestry.channel.ICMPPingMsg;
import ostore.tapestry.channel.ICMPPingRouteMsg;
import ostore.tapestry.channel.ICMPPongMsg;
import ostore.util.NodeId;

/**
 * Registered in MainClass and listens for packets containing a
 * CreateObjectRequest Message.
 *      
 * @author   Jason Lee <jlee81@cs.berkeley.edu>
 */

public class ICMPAnalyzer extends PacketAnalyzer implements Runnable {
    protected TwoKeyHashtable ping_edge_info;
    protected TwoKeyHashtable pong_edge_info;

    protected Thread del_edge_thread;
    
    public ICMPAnalyzer () {
	ping_edge_info = new TwoKeyHashtable();
	pong_edge_info = new TwoKeyHashtable();
	del_edge_thread = new Thread(this);
	del_edge_thread.start();
    }

    public void run() {
	EdgeInfo tempEdge = null;
	while (del_edge_thread == Thread.currentThread()) {
	    long currentTime = Calendar.getInstance().getTime().getTime();
	    for(Enumeration edges = ping_edge_info.elements(); 
		edges.hasMoreElements();) {
		tempEdge = (EdgeInfo)edges.nextElement(); 
		if(currentTime - MainClass.edgeEventLongevity > 
			tempEdge.getTime()) {
		    if(tempEdge.getInstances() > 1) {
		        tempEdge.instances--;
			MainClass.displayManager.modifyEdgeWidth(tempEdge);
		    } else {
			MainClass.objectMaintainer.triggerEdgeDeletedEvent(new
			  EdgeEvent(this, tempEdge, Calendar.getInstance().getTime()));
			ping_edge_info.remove(tempEdge.getSource(), tempEdge.getDest());
		    }	
		}
	    }
	    for(Enumeration edges = pong_edge_info.elements(); 
		edges.hasMoreElements();) {
		tempEdge = (EdgeInfo)edges.nextElement(); 
		if(currentTime - MainClass.edgeEventLongevity > 
			tempEdge.getTime()) {
		    if(tempEdge.getInstances() > 1) {
		        tempEdge.instances--;
			MainClass.displayManager.modifyEdgeWidth(tempEdge);
		    } else {
			MainClass.objectMaintainer.triggerEdgeDeletedEvent(new
			  EdgeEvent(this, tempEdge, Calendar.getInstance().getTime()));
			pong_edge_info.remove(tempEdge.getSource(), tempEdge.getDest());
		    }	
		}
	    }
	}
    }
    
    public void packetReceived(PacketEvent e) {
	int offset[] = {0};
	int typeData = -1;
	ostore.tapestry.impl.RouteMsg rt_msg = null;

	//TODO: implement later
	if(!e.hasPayload()) return;

	if(e.getPacket() instanceof RouteMsg) {
	    rt_msg = (RouteMsg) e.getPacket(); 
	}

	try { 
	if(rt_msg != null && rt_msg.user_data() != null 
		&& (rt_msg.user_data() instanceof ICMPPingMsg 
		    || rt_msg.user_data() instanceof ICMPPongMsg
		    || rt_msg instanceof ICMPPingRouteMsg)) {
	    NodeId sender = null; 
	    NodeId recp  = null; 
	    boolean isPing = true;

	    if(rt_msg.user_data() instanceof ICMPPongMsg) isPing = false; 
		
	    sender =  e.getSender();
	    recp  =  e.getRecipient(); 

	    if(sender != null && recp != null) {
		EdgeInfo edgeInfo = null;
		if(ping_edge_info.get(sender,recp) != null ||
		   pong_edge_info.get(sender,recp) != null) {
		   //DO NOTHING
		 } else if(isPing) {
		     edgeInfo = new EdgeInfo(sender, recp, "ostore.tapestry.channel.ICMPPingMsg", Calendar.getInstance().getTime().getTime());
		    ping_edge_info.put(sender, recp, edgeInfo); 
		    MainClass.objectMaintainer.triggerEdgeCreatedEvent(new 
			EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime()));

		    //TESTING
		    edgeInfo = (EdgeInfo) ping_edge_info.get(sender, recp);
		    edgeInfo.instances += 3;
		    MainClass.displayManager.modifyEdgeWidth(edgeInfo);
		    //TESTING
			
		} else {
		    edgeInfo = new EdgeInfo(sender, recp, "ostore.tapestry.channel.ICMPPongMsg", Calendar.getInstance().getTime().getTime());
		    pong_edge_info.put(sender, recp, edgeInfo); 
		    MainClass.objectMaintainer.triggerEdgeCreatedEvent(new 
			EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime()));
		}
	    }
	}
	} catch (Exception qse) {
		qse.printStackTrace();
	}
	
    }
}
