/**
 * RemoteReplicaAnalyzer.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.PacketAnalyzers;

import Sonar.*;
import Sonar.util.*;
import Sonar.event.*;
import Sonar.Dialog.*;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.zip.*;

import ostore.tapestry.impl.RouteMsg;
import ostore.replica.RemoteReplicaCreateReq;
import ostore.replica.RemoteReplicaUpdateReq;
import ostore.util.NodeId;

import ostore.apps.visdemo.VisUp;

//SANDSTORM
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.NodeId;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;

import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.internal.SinkProxy;
/**
 *      
 * @author   Jason Lee <jlee81@cs.berkeley.edu>
 */

public class RemoteReplicaAnalyzer extends PacketAnalyzer implements EventHandlerIF, ProfilableIF, SingleThreadedEventHandlerIF {

    protected TwoKeyHashtable edge_info;
    protected long edgeEventLongevity = 2 * 1000;
    protected boolean _show_along_tapestry = false;

    public void inst () {
	edge_info = new TwoKeyHashtable();
    }

    public void checkEdges() {
	final String tag = class_tag + ".checkEdges";
	EdgeInfo tempEdge = null;
	long currentTime = Calendar.getInstance().getTime().getTime();
	try {
	    for(Enumeration edges = edge_info.elements(); edges.hasMoreElements();) {
		tempEdge = (EdgeInfo)edges.nextElement(); 
		if (currentTime - edgeEventLongevity > tempEdge.getTime() 
			&& currentTime - edgeEventLongevity > tempEdge.getTime()
			&& tempEdge.getInstances() == -1 ) {
		    edge_info.remove(tempEdge.getSource(),
			    tempEdge.getDest());
		    /*
		    if(DEBUG) Debug.printtagln(tag, "resetting the source, dest nodes");
		    _classifier.dispatch(new NodeEvent(this, tempEdge.getSource(), 
				Calendar.getInstance().getTime(), NodeEvent.ID,
				DisplayManager.NEVER_SET));
		    _classifier.dispatch(new NodeEvent(this, tempEdge.getDest(),
				Calendar.getInstance().getTime(), NodeEvent.ID,
				DisplayManager.NEVER_SET));
		    */

		} else if(currentTime - edgeEventLongevity > 
			tempEdge.getTime() && tempEdge.getInstances() >= 0) {
		    if(tempEdge.getInstances() > 1) {
			tempEdge.instances--;
			_classifier.dispatch(new EdgeEvent(this, tempEdge,
				    Calendar.getInstance().getTime(), 
				    EdgeEvent.MODIFY_WIDTH));
		    } else {
			_classifier.dispatch(new EdgeEvent(this, tempEdge,
				    Calendar.getInstance().getTime(), EdgeEvent.DELETE));
			tempEdge.instances = -1;
		    }	
		}
	    }
	} catch (SinkFullException e) {
	    e.printStackTrace();
	}

	Signal.getSignal(_self_node_id).register(Signal.ALRM, stageName, _this_sink);
	Signal.getSignal(_self_node_id).alarm(stageName, ALARM_SLEEP);
    }

    public void handleMsg (VisUp son_msg) {
	final String tag = class_tag + "handleMsg";

	/* int offset[] = {0};
	int typeData = -1; */
	//RouteMsg msg = null;
	NodeId sender = null; 
	NodeId recp  = null; 
	String msg_type = "";

	/*
	if(e.hasPayload()) {
	    if(e.getPacket() instanceof RouteMsg) {
		msg = (RouteMsg) e.getPacket(); 
	    }
	    try { 
		if(msg != null && msg.user_data() != null 
		    && (msg.user_data() instanceof RemoteReplicaCreateReq
			|| msg.user_data() instanceof RemoteReplicaUpdateReq)) {
		    sender = MainClass.objectMaintainer.guid_to_node(msg.src);
		    recp = MainClass.objectMaintainer.guid_to_node(msg.dest);
		} else return;
	    } catch (Exception qse) {
		qse.printStackTrace();
	    }
	} else {
	}
	*/
	if(son_msg.hasPayload()) return;
	    
	if(son_msg.getMsgName().equals("ostore.replica.RemoteReplicaCreateReq") 
		|| son_msg.getMsgName().equals("ostore.replica.RemoteReplicaUpdateReq")) {
	    msg_type = son_msg.getMsgName();
	} else return;

	//If only sending strings containing the queried msg type	
	if(_hash_to_node_map == null || _node_to_hash_map == null) return;
	sender = (NodeId) _hash_to_node_map.get(son_msg.getSource());
	recp   = (NodeId) _hash_to_node_map.get(son_msg.getDest());

	//Hop to hop (then the sender/recp is overwritten)
	if(_show_along_tapestry) {
	    sender = son_msg.getSender();
	    recp = son_msg.getRecp(); 
	}

	try {
	    _classifier.dispatch(new NodeEvent(this, recp, Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.REPLICA));

	    if(DEBUG) Debug.printtagln(tag, "received a "+msg_type+" from "+sender+" to "+recp);
	    //trigger a new edge evnet
	    if(sender != null && recp != null) {
		EdgeInfo edgeInfo = null;
		if((edgeInfo = (EdgeInfo) edge_info.get(sender, recp)) != null) {
		    edgeInfo.instances++;
		    edgeInfo.modifyTime(Calendar.getInstance().getTime().getTime());
		    _classifier.dispatch(new EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime(), EdgeEvent.MODIFY_WIDTH));
		} else {
		    edgeInfo = new EdgeInfo(sender, recp, msg_type, 
			    Calendar.getInstance().getTime().getTime());
		    edge_info.put(sender, recp, edgeInfo); 
		    _classifier.dispatch(new EdgeEvent(this, edgeInfo, Calendar.getInstance().getTime(), EdgeEvent.CREATE));
		}
	    }
	} catch (SinkFullException e) {
	    e.printStackTrace();
	}
    }

    /**
     * SANDSTORM 
     */
    private static final String class_tag = "RemoteReplicaAnalyzer";
    private static final boolean DEBUG = true;

    protected Map _node_to_hash_map = null;
    protected Map _hash_to_node_map = null;

    public void handleEvent (QueueElementIF item) throws EventHandlerException {
	final String tag = class_tag + ".handleEvent";
	if (item instanceof VisUp) {
	    handleMsg ((VisUp) item);
	} else if (item instanceof Signal.AlarmSignal) {
	    checkEdges();
	} else if (item instanceof GuidMapEvent) {
	    _node_to_hash_map = ((GuidMapEvent) item).getNodeToHash();
	    _hash_to_node_map = ((GuidMapEvent) item).getHashToNode();
	}
    }

    public void handleEvents (QueueElementIF items[]) throws EventHandlerException {
	final String tag = class_tag + ".handleEvents";
	for(int i=0; i<items.length; i++ ) 
	    handleEvent(items[i]);
    }

    public void init(ConfigDataIF config) throws Exception {
	final String tag = class_tag + ".init";

	//Sets all the variables to initial values
	inst();

	Filter filter = null;
	String messageTypes[] = {"ostore.apps.visdemo.VisUp",
				 "Sonar.event.GuidMapEvent"};
	
	if (DEBUG) Debug.turnOn(class_tag);
	if (DEBUG) Debug.printtagln (tag, "Debug turned on ");

	_self_node_id = new NodeId(config.getString("node_id"));
	_classifier = Classifier.getClassifier ( _self_node_id);

	stageName = config.getStage().getName();
	if(config.getBoolean("Profile")) {
	    config.getManager().getProfiler().add("Sonar.PacketAnalyzers.RemoteReplicaAnalyzer", this);
	    SinkIF sink = config.getStage().getSink();
	    _this_sink = new SinkProxy(sink, config.getManager(), 
		    config.getStage().getWrapper());
	} else 
	    _this_sink = config.getStage().getSink();

	for(int i = 0; i < messageTypes.length; i++) {
	    filter = new Filter();
	    if(!filter.requireType(Class.forName (messageTypes[i])))
		BUG ( tag + ": could not require type==" +
			messageTypes[i] + ".");
	    _classifier.subscribe (filter, _this_sink);
	}

	Signal.getSignal(_self_node_id).register(Signal.ALRM, stageName, _this_sink);
	Signal.getSignal(_self_node_id).alarm(stageName, ALARM_SLEEP);
    }
    private final long ALARM_SLEEP = 150;

    // methods for ProfilableIF
    public int profileSize()
    {
	return _this_sink.size();
    }
    // end methods for ProfilableIF

    // methods for EventHandlerIF
    public void destroy () throws Exception { }
    // end methods for EventHandlerIF

    private static final void BUG (String msg) {
	Carp.die (msg + " This is a bug in " + class_tag + ". " +
		"Please let Jason know about it ASAP." );
    }

    private NodeId _self_node_id;
    private String stageName;
    private Classifier _classifier;
    private SinkIF _this_sink;
}
