/**
 * RoutingTableAnalyzer.java 
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.PacketAnalyzers;

import Sonar.*;
import Sonar.util.*;
import Sonar.event.*;
import Sonar.Dialog.*;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.zip.*;

import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.QSString;
import ostore.util.QSVector;
import ostore.tapestry.impl.RoutingTable;
import ostore.tapestry.impl.RouteEntry;
import ostore.apps.visdemo.RoutingTableMessage;
import ostore.apps.visdemo.QTreeStateMsg;
import ostore.apps.visdemo.VisUp;

import ostore.network.CloseConnectionMsg;

//SANDSTORM
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.NodeId;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;

import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.internal.SinkProxy;

import seda.sandStorm.api.StagesInitializedSignal;

/**
 * RoutingTableAnalyzer is registered in MainClass to listen for packets,
 * of which it handles the RoutingTableMessages (ostore.apps.visdemo). It
 * could "delete" an expired node, update a node's SHA1Hash, draw the edges
 * between nodes as well as the paint arrows showing the connectivity of each
 * link. Furthermore, it is the one class that registers all the types in the
 * Sonar package (but could be placed in MainClass.java instead).
 *      
 * @author	Jason Lee <jlee81@cs.berkeley.edu>
 * @see		Sonar.PacketAnalyzers.PacketAnalyzer
 */

public class RoutingTableAnalyzer extends PacketAnalyzer implements EventHandlerIF, ProfilableIF, SingleThreadedEventHandlerIF {
    protected Hashtable proprietary_node_info;
    protected boolean node_deletion = false;
    protected long expiration_time = 2 * 60 * 1000; //2 minutes
    private long ALARM_SLEEP = 500; //deletion of nodes
    protected MenuManager menu_manager;

    protected QSVector route_msgs;
    public void setNodeDeletion(boolean val) { node_deletion = val; }

    protected static Map nodeToHashMap;
    protected static Map hashToNodeMap;

    //public RoutingTableAnalyzer() {
    public void inst() {
	ostore.util.Types.registerAllTypes();
	registerTypes();

	proprietary_node_info = new Hashtable();
	menu_manager = new MenuManager();
	nodeToHashMap = new HashMap();
	hashToNodeMap = new HashMap();

	//HACK
	route_msgs = new QSVector();
	route_msgs.add(new QSString("ostore.rp.CreateObjectReqMsg"));
	route_msgs.add(new QSString("ostore.rp.InnerRingCreateReqMsg"));
	route_msgs.add(new QSString("ostore.rp.ObjectCreateReqMsg"));
	route_msgs.add(new QSString("ostore.agree.ForwardMsg"));
	route_msgs.add(new QSString("ostore.agree.ResultMsg"));
	route_msgs.add(new QSString("ostore.archive.dissemination.DisseminateFragMsg"));
    }

    protected void registerTypes() {
	String[] types = { 
	    "ostore.network.CloseConnectionMsg",

	    "ostore.apps.visdemo.RoutingTableMessage",
	    "ostore.apps.visdemo.QTreeStateMsg",
	    "ostore.apps.visdemo.QTreeUpMsg",
	    "ostore.apps.visdemo.VisUp",

	    "ostore.tapestry.impl.RoutingTable", 
	    "ostore.tapestry.impl.RouteEntry", 
	    "ostore.tapestry.impl.RouteMsg",
	    "ostore.util.SecureHash",
	    "ostore.util.QSVector",

	    //agree
	    "ostore.agree.ForwardMsg",
	    "ostore.agree.ForwardMsg",
	    "ostore.agree.PreprepareMsg",
	    "ostore.agree.PrepareMsg",
	    "ostore.agree.CommitMsg",
	    "ostore.agree.ResultMsg",

	    //update
	    "ostore.update.GuidVersionMap",
	    "ostore.tapestry.channel.ICMPPingMsg",
	    "ostore.tapestry.channel.ICMPPingRouteMsg",
	    "ostore.tapestry.channel.ICMPPongMsg"
	};
	for(int i = 0 ; i < types.length; i++ ) {
	    try { 
		TypeTable.register_type(types[i]);
	    } catch (Exception e) {

	    }
	}
    }

    /*
     * Checks the expiration time and could mark a given node as expired or 
     * not responding.
     */
    protected void checkNodes() {
	Signal.getSignal(_self_node_id).register(Signal.ALRM, stageName, _this_sink);
	Signal.getSignal(_self_node_id).alarm(stageName, ALARM_SLEEP);

	NodeInfo node_info = null;
	if(!node_deletion) return;

	long currentTime = Calendar.getInstance().getTime().getTime();
	for(Enumeration nodes = proprietary_node_info.elements();
		nodes.hasMoreElements();) {
	    node_info = (NodeInfo) nodes.nextElement();
	    try { 
		if(!node_info.isNotResponding() && !node_info.isDeleted()
			&& currentTime - 0.5*expiration_time > 
			node_info.getUpdateTime()) {
		    _classifier.dispatch(new NodeEvent(this, node_info.getNodeId(), Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.NON_RESPONSIVE));
		    node_info.markNotResponding(true);
		} else if(!node_info.isDeleted() && 
			currentTime - expiration_time > 
			node_info.getUpdateTime()) {
		    _classifier.dispatch(new NodeEvent(this, node_info, Calendar.getInstance().getTime(), NodeEvent.DELETE));
		    //proprietary_node_info.remove(node_info.getNodeId());
		    node_info.markDeleted(true);
		}
	    } catch ( SinkFullException e ) {
		e.printStackTrace();
	    }
	}

    }

    /**
     * Guids to NodeId Mapping
     */
  public static synchronized SecureHash node_to_guid(NodeId node) {
    return (SecureHash) nodeToHashMap.get(node);
  }
  public static synchronized NodeId guid_to_node(SecureHash guid) {
    return (NodeId) hashToNodeMap.get(guid);
  }

  /*
   * Attribute a node with a SHA1Hash and vice versa
   */
  public static synchronized boolean associate_to_guid(NodeId node, SecureHash element) {
      final String tag = class_tag + ".associate_to_guid";
      NodeId oldNode = null;
      SecureHash oldSHash = null;
     
      /*
       * If both the node id and the SHA1Hash have never been seen before
       */
      if((oldSHash = (SecureHash) nodeToHashMap.get(node)) == null && 
	      (oldNode = (NodeId) hashToNodeMap.get(element)) == null) {
	  nodeToHashMap.put(node, element);
	  hashToNodeMap.put(element, node);
	  return true;
      } else if(oldNode == null) {
	  oldNode = (NodeId) hashToNodeMap.get(oldSHash);
	  //If we have an assoc node id with the given SHA1Hash
	  if(oldNode != null) { 
	      if(oldSHash.equals(element)) { //If the SHA1Hash's are the same
	      	  if(DEBUG) Debug.printtagln(tag, oldNode+" for (same) old: "+
		  oldSHash+ " vs. new: "+element);
	      } else { //If the SHA1Hash's are different
	      	  if(DEBUG) Debug.printtagln(tag, oldNode+" for (diff) old: "+
		  oldSHash+ " vs. new: "+element +" and old nodeid reset");
		  //Remove the old mapping
	  	  nodeToHashMap.remove(node); 
	  	  nodeToHashMap.put(node, element);
	      }
	  }
	  //hashToNodeMap.remove(element); //Keep both elements
	  hashToNodeMap.put(element, node);
      } else if(oldSHash == null) {
	  if(DEBUG) Debug.printtagln(tag,"null hash, but node's there");
	  //nodeToHashMap.remove(node);
      }
      if(DEBUG) Debug.printtagln(tag, "Attempting to assign existing nodeid "+oldNode+" ," + oldSHash + " to:"+node+" ,"+element);
      return false;
  }

  /*
   * Replaces the current mapping of node id to SHA1Hash to a new hash value
   */
  public synchronized void replace_with_new_guid (NodeId node, SecureHash element) {
      final String tag = class_tag + ".replace_with_new_guid";
    SecureHash oldHash = (SecureHash) nodeToHashMap.get(node);

    nodeToHashMap.remove(node);
    nodeToHashMap.put(node, element);

    hashToNodeMap.remove(oldHash);
    hashToNodeMap.put(element, node);
    if(DEBUG) Debug.printtagln(tag, "Replaced ==("+oldHash.equals(element)+") "+oldHash+" with "+element+ " for "+node.port());
  }
    
  /**
   * Handles a routing table message
   */
    public void handleRTM ( RoutingTableMessage msg ) {
	final String tag = class_tag + ".handleRTM";

	if(DEBUG) Debug.printtagln (tag, "received a routing table");

	RoutingTable rout_tbl = msg._routing_table;
	NodeId   self_node_id = rout_tbl.get_self_node_id(); 
	NodeInfo self_info = (NodeInfo) proprietary_node_info.get(self_node_id);

	if(self_info == null) {
	    /** New Node **/
	    self_info = new NodeInfo(rout_tbl);
	    proprietary_node_info.put(self_node_id, self_info);
	    if(DEBUG) Debug.printtagln (tag, "new node : "+self_node_id+ ", "+ self_info.get_self_guid());
	    try { 
		_classifier.dispatch(new NodeEvent(this,self_info, Calendar.getInstance().getTime()));
	    } catch (seda.sandStorm.api.SinkFullException e) {
		Debug.printtagln (tag, ": could not dispatch a node event");
	    }

	     
	    if(!associate_to_guid(self_node_id, self_info.get_self_guid())) {
		if(DEBUG) Debug.printtagln(tag, "RTA> omOS has already seen the self node id");
	    }

	    //TODO register messages
	    //SonarReqMessage msg = new SonarReqMessage(self_node_id, false, new ostore.util.SHA1Hash(), SonarReqMessage.ROUTEMSG, route_msgs);
	    //MainClass.packetSender.sendMessage(msg);
	} else if(self_info.isNotResponding() || self_info.isDeleted()) {
	    if(DEBUG) Debug.printtagln ( tag, "reviving (resetting) node: "+self_node_id);
	    /* 
	    try { 
		_classifier.dispatch(new NodeEvent(this, self_info.getNodeId(), Calendar.getInstance().getTime(), NodeEvent.ID, DisplayManager.NEVER_SET));
	    } catch(SinkFullException e) {
		e.printStackTrace();
	    }
	    */

	    if(self_info.isDeleted()) {
		try { 
		    _classifier.dispatch(new NodeEvent(this,self_info, Calendar.getInstance().getTime()));
		} catch (seda.sandStorm.api.SinkFullException e) {
		    Debug.printtagln (tag, "could not dispatch a node event");
		}
	    }
	    self_info.markDeleted(false); //Also marks _notResponding as false
	    self_info.updateTime(Calendar.getInstance().getTime().getTime());

	    SecureHash guid = rout_tbl.node_id_to_guid(self_node_id);
	    if(self_info.get_self_guid() != null && !self_info.get_self_guid().equals(guid)) {
		replace_with_new_guid(self_node_id, guid);
		self_info.set_self_guid(guid);
	    }
	} else {
	    if(DEBUG) Debug.printtagln (tag, "received a routing table from:"+self_node_id+", "+rout_tbl.node_id_to_guid(self_node_id));
	    self_info.updateTime(Calendar.getInstance().getTime().getTime());
	    SecureHash guid = rout_tbl.node_id_to_guid(self_node_id);

	    if(self_info.get_self_guid() != null && !self_info.get_self_guid().equals(guid)) {
		replace_with_new_guid(self_node_id, guid);
		self_info.set_self_guid(guid);
	    }
	}

	/*
	 * The following goes through the routingtable looking for new entries.
	 * However, it will not update the time stamps on those nodes (that's only performed
	 * if the node sends a routing table message too).
	 */
	RouteEntry[][] routing_entries = rout_tbl.get_table();
	NodeId node_entry = null;
	Vector neighbor_entries = new Vector();

	for(int i=0; i < routing_entries.length; i++) {
	    for(int j=0; j < routing_entries[0].length; j++) { //It's a rectangular matrix
		RouteEntry entry = routing_entries[i][j];
		if(entry != null) {
		    for(int k=0; k < entry.size(); k++) {
			node_entry = entry.getAtIndex(k);
			if(node_entry != null && !node_entry.equals(self_node_id)) {
			    NodeInfo node_entry_info = (NodeInfo) proprietary_node_info.get(node_entry);
			    neighbor_entries.addElement(node_entry);
			    if(!self_info.get_neighbors().contains(node_entry)) {
				self_info.get_neighbors().addElement(node_entry);
			    }
			    //If the node does not exist
			    if(node_entry_info == null) {
				proprietary_node_info.put(node_entry, new NodeInfo(node_entry,
					    rout_tbl.node_id_to_guid(node_entry)));
				NodeInfo neighbor = new NodeInfo(node_entry);
				/* ask neighbor for its routing table */
				//if(DEBUG) Debug.printtagln(tag, "RTA> req rout. table from "+node_entry.toString());
				//TODO if election, dont bother
				//QTreeStateMsg vis_msg = new QTreeStateMsg(node_entry,false, QTreeStateMsg.ROUTING, _self_node_id);
				//automatically returns routing table
				try {
				    //_classifier.dispatch(vis_msg);
				    _classifier.dispatch(new NodeEvent(this, neighbor, Calendar.getInstance().getTime()));
				    _classifier.dispatch(new EdgeEvent(this, new EdgeInfo(self_node_id, node_entry), Calendar.getInstance().getTime()));
				} catch (seda.sandStorm.api.SinkFullException e) {
				    Debug.printtagln(tag, "could not dispatch a visdemo state message/node or edge event");
				}

				/* register messages
				//SonarReqMessage msg2 = new SonarReqMessage(node_entry, false, new ostore.util.SHA1Hash(), SonarReqMessage.ROUTEMSG, route_msgs);
				//MainClass.packetSender.sendMessage(msg2);
				*/
				if(!associate_to_guid(node_entry, rout_tbl.node_id_to_guid(node_entry))) {
				    if(DEBUG) Debug.printtagln(tag, "RTA> already seen this node entry: "+node_entry);
				}

			    } else if(!node_entry.equals(self_node_id) && !node_entry_info.isDeleted()) {
				//if the node exists, and hasn't been deleted
				try {
				    _classifier.dispatch(new EdgeEvent(this, new EdgeInfo(self_node_id, node_entry), Calendar.getInstance().getTime()));
				} catch (seda.sandStorm.api.SinkFullException e) {
				    Debug.printtagln(tag, "could not dispatch an edge event");
				}
			    }
			}
			//MainClass.out(self_info.self_guid + "> neighbor:"+MainClass.objectMaintainer.node_to_guid(key));
		    }
		}
	    }
	}

	/* To check for any 'removed' edges from the routing table, i.e. any entries
	 * that were in an older routing table, but no longer exist
	 */
	boolean modifiedRoutingTable = false;
	for(Enumeration elms = self_info.get_neighbors().elements(); elms.hasMoreElements();) {
	    node_entry = (NodeId) elms.nextElement();
	    if(!neighbor_entries.contains(node_entry)) {
		self_info.get_neighbors().remove(node_entry);

		try { 
		    _classifier.dispatch(new EdgeEvent(this, new EdgeInfo(self_node_id, node_entry), Calendar.getInstance().getTime(), EdgeEvent.DELETE));
		} catch (seda.sandStorm.api.SinkFullException e) {
		    Debug.printtagln(tag, "could not dispatch a delete edge event");
		}
		//Means a new experiment has been run and routing tables have been modified
		//and the hack below needs to be run
		modifiedRoutingTable = true;
	    }
	}

	//TODO create an event to recalculate the positions
	if(modifiedRoutingTable) {
	    /*
	    GridLocationAnalyzer gla = (GridLocationAnalyzer) MainClass.locationAnalyzer;
	    if(gla.layoutMode == gla.CIRCULAR) {
		gla.makeConcentric();
	    } else if(gla.layoutMode == gla.TREE) {
		gla.makeTree();
	    }
	    */
	    try {
		_classifier.dispatch(new UpdateGraphEvent(this));
	    } catch (SinkFullException e) {
		e.printStackTrace();
	    }
	}
    }

    /*
     * Primarily used in the GridLocationAnalyzer for the concentric layout.
     * Just returns the neighbors of this given node, i.e. nodes that are in
     * its routing table.
     */
    public Vector getNeighbors(NodeId node) {
	NodeInfo node_info = null;
	if((node_info = (NodeInfo) proprietary_node_info.get(node)) != null) {
	    return node_info.get_neighbors();
	}
	return null;
    }

    /*
     * RTA should be the only analyzer that informs all other node listeners
     * that a node has been deleted/created.
     */
    public void nodeCreated(NodeEvent e) { } 
    public void nodeDeleted(NodeEvent e) { }

    public void registerMessages(QSVector vector, int type) {
	/*
	NodeInfo node_info = null;
	for(Enumeration nodes = proprietary_node_info.elements();
		nodes.hasMoreElements();) {
	    node_info = (NodeInfo) nodes.nextElement();
	    MainClass.packetSender.sendMessage(new SonarReqMessage
		    (node_info.getNodeId(), false, new ostore.util.SHA1Hash(),
		     type, vector));
	}
	*/
    }

    //Send a message to each node to unregister (in QTree)
    public void exit() {
	final String tag = class_tag + ".exit";

	NodeInfo node_info = null;
	for(Enumeration nodes = proprietary_node_info.elements();
		nodes.hasMoreElements();) {
	    node_info = (NodeInfo) nodes.nextElement();
	    try {
		_classifier.dispatch(new QTreeStateMsg(node_info.getNodeId(), false, QTreeStateMsg.CLOSE, _self_node_id, 0));
	    } catch (SinkFullException e) {
		e.printStackTrace();
	    }
	    
	    //MainClass.packetSender.sendMessage(new QTreeStateMsg(node_info.getNodeId(), false, QTreeStateMsg.CLOSE, new NodeId(MainClass.port, MainClass.hostAddress)));
	    //extra below?
	    //MainClass.packetSender.sendMessage(new CloseConnectionMsg(new NodeId(MainClass.port, MainClass.hostAddress), node_info.getNodeId()));
	}
	if(DEBUG) Debug.printtagln(tag, "sent a close connection message to all connected nodes");
	
    }

    /**
     * SANDSTORM 
     */
    private static final String class_tag = "RoutingTableAnalyzer";
    private static final boolean DEBUG = false;

    private boolean sent_options_menu = false;

    public void handleEvent (QueueElementIF item) throws EventHandlerException {
	final String tag = class_tag + ".handleEvent";
	if (item instanceof VisUp) {
	    VisUp msg = (VisUp)item;
	    if(msg.hasPayload() && msg.getMsg() instanceof RoutingTableMessage)
		handleRTM((RoutingTableMessage) msg.getMsg());
	} else if (item instanceof StagesInitializedSignal) {
	    try {
		//if(DEBUG) Debug.printtagln(tag, "registering rta as a panel contributor");
		_classifier.dispatch(new MainMenuEvent(menu_manager.mainMenu));
		_classifier.dispatch(new OptionsMenuEvent(this));

		//map information
		_classifier.dispatch(new GuidMapEvent(this, nodeToHashMap, hashToNodeMap));
	    } catch (SinkFullException e) {
		e.printStackTrace();
	    }
	} else if (item instanceof Signal.AlarmSignal) {
	    checkNodes();
	}
    }

    public void handleEvents (QueueElementIF items[]) throws EventHandlerException {
	final String tag = class_tag + ".handleEvents";
	for(int i=0; i<items.length; i++ ) 
	    handleEvent(items[i]);
    }

    public void init(ConfigDataIF config) throws Exception {
	final String tag = class_tag + ".init";

	//Sets all the variables to initial values
	inst();

	Filter filter = null;
	String messageTypes[] = {"ostore.apps.visdemo.VisUp",
				 "seda.sandStorm.api.StagesInitializedSignal"};

	if (DEBUG) Debug.turnOn(class_tag);
	if (DEBUG) Debug.printtagln (tag, "Debug turned on ");

	_self_node_id = new NodeId(config.getString("node_id"));
	_classifier = Classifier.getClassifier ( _self_node_id);

	stageName = config.getStage().getName();
	if(config.getBoolean("Profile")) {
	    config.getManager().getProfiler().add("Sonar.PacketAnalyzers.RoutingTableAnalyzer", this);
	    SinkIF sink = config.getStage().getSink();
	    _this_sink = new SinkProxy(sink, config.getManager(), 
		    config.getStage().getWrapper());
	} else 
	    _this_sink = config.getStage().getSink();

	for(int i = 0; i < messageTypes.length; i++) {
	    filter = new Filter();
	    if(!filter.requireType(Class.forName (messageTypes[i])))
		BUG ( tag + ": could not require type==" +
			messageTypes[i] + ".");
	    _classifier.subscribe (filter, _this_sink);
	}

	Signal.getSignal(_self_node_id).register(Signal.ALRM, stageName, _this_sink);
	Signal.getSignal(_self_node_id).alarm(stageName, ALARM_SLEEP);
    }

    // methods for ProfilableIF
    public int profileSize()
    {
	return _this_sink.size();
    }
    // end methods for ProfilableIF

    // methods for EventHandlerIF
    public void destroy () throws Exception { }
    // end methods for EventHandlerIF

    private static final void BUG (String msg) {
	Carp.die (msg + " This is a bug in " + class_tag + ". " +
		"Please let Jason know about it ASAP." );
    }

    private NodeId _self_node_id;
    private String stageName;
    private Classifier _classifier;
    private SinkIF _this_sink;

    /**
     * Options menu
     *
     */
    public void showOptionsDialog() {
	StandardDialog newDialog = new StandardDialog(new OptionsPanel()); 
	newDialog.show(); 
    } 

    public ActivePanel getOptionsPanel() { 
	return new OptionsPanel(); 
    }  

    /** 
     * RoutingTableAnalyzerPropertiesPanel.
     * This class is an ActivePanel and should have all the information 
     * in GUI form that this class stores with respect to EDGES 
     * It will be displayed automatically with showOptionsDialog 
     */
    public class OptionsPanel extends Sonar.Dialog.ActivePanel 
    {		 
	public OptionsPanel() 
	{ 
	    tabTitle = "Routing";//this will be the title of the tab 
	    //{{INIT_CONTROLS 
	    setLayout(null); 
	    //Insets ins = getInsets(); 
	    setSize(307,168); 
	    /*
	    JCheckBox1.setToolTipText("Check this if you want to display the tapestry edges between nodes");
	    JCheckBox1.setText("Tapestry Edges"); 
	    add(JCheckBox1); 
	    JCheckBox1.setBounds(36,20,168,24); 

	    JCheckBox2.setToolTipText("Check this if you want to show the directional arrows on the tapestry edges");
	    JCheckBox2.setText("Arrows for the Tapestry Edges");
	    add(JCheckBox2); 
	    JCheckBox2.setBounds(36,50,200,24); */

	    JCheckBox3.setToolTipText("Check this if you want nodes to be displayed indefinitely");
	    JCheckBox3.setText("Node Expiration");
	    add(JCheckBox3);
	    JCheckBox3.setBounds(36, 80, 200, 24);

	    //}} 
	    //{{REGISTER_LISTENERS 
	    //}} 
	} 

	//{{DECLARE_CONTROLS 
	//javax.swing.JCheckBox JCheckBox1 = new javax.swing.JCheckBox(); 
	//javax.swing.JCheckBox JCheckBox2 = new javax.swing.JCheckBox(); 
	javax.swing.JCheckBox JCheckBox3 = new javax.swing.JCheckBox();
	//}} 

	public void applyChanges()//this function will be called when the apply button is hit 
	{ 
	    toggleNodeDeletion();
	} 

	public void initializeDisplayValues()//this function will be called when the panel is first shown 
	{ 
	    //JCheckBox1.setSelected(MainClass.displayManager.showTapestryEdges);
	    //JCheckBox2.setSelected(MainClass.displayManager.showTapestryArrows);
	    JCheckBox3.setSelected(node_deletion);
	} 

	public void toggleNodeDeletion() {
	    if(JCheckBox3.isSelected() && !node_deletion) {
		node_deletion = true;
	    } else if(!JCheckBox3.isSelected() && node_deletion) {
		node_deletion = false;
	    }
	}

    }	           

    /** MENU MANAGER.
     * This class creates and holds the menu that controls this 
     * PacketAnalyzer.  It returns the menu to whoever wants 
     * to display it and it also handles all events on the menu 
     */
    protected class MenuManager implements Serializable, ActionListener, ItemListener 
    { 
	//{{DECLARE_CONTROLS 
	JMenu mainMenu = new JMenu(); 
	JCheckBoxMenuItem packetProcessingCheckBox = new JCheckBoxMenuItem(); 
	JCheckBoxMenuItem backgroundProcessingCheckBox = new JCheckBoxMenuItem(); 
	JSeparator separator1 = new JSeparator(); 
	JMenuItem propertiesItem = new JMenuItem(); 
	JSeparator separator2 = new JSeparator(); 
	JMenu serializeMenu = new JMenu(); 
	JMenuItem saveNodesItem = new JMenuItem(); 
	JMenuItem loadNodesItem = new JMenuItem(); 
	JMenuItem saveEdgesItem = new JMenuItem(); 
	JMenuItem loadEdgesItem = new JMenuItem(); 
	JSeparator separator3 = new JSeparator(); 
	JMenu paintMenu = new JMenu(); 
	JCheckBoxMenuItem paintNodesItem = new JCheckBoxMenuItem(); 
	JCheckBoxMenuItem paintEdgesItem = new JCheckBoxMenuItem(); 
	JCheckBoxMenuItem paintScreenItem = new JCheckBoxMenuItem(); 
	//}} 

	public MenuManager() 
	{ 
	    //{{INIT_CONTROLS 
	    mainMenu.setText("Routing Table Analyzer"); 
	    mainMenu.setActionCommand("Routing Table Analyzer"); 

	    packetProcessingCheckBox.setSelected(true); 
	    packetProcessingCheckBox.setText("Packet Processing"); 
	    packetProcessingCheckBox.setActionCommand("Packet Processing"); 
	    mainMenu.add(packetProcessingCheckBox); 

	    /*
	       backgroundProcessingCheckBox.setSelected(true); 
	       backgroundProcessingCheckBox.setText("Background Processing"); 
	       backgroundProcessingCheckBox.setActionCommand("Background Processing"); 
	       mainMenu.add(backgroundProcessingCheckBox); 
	       */

	    mainMenu.add(separator1); 
	    propertiesItem.setText("Options"); 
	    propertiesItem.setActionCommand("Options"); 
	    mainMenu.add(propertiesItem); 

	    /*
	       mainMenu.add(separator2); 
	       paintMenu.setText("Painting"); 
	       paintMenu.setActionCommand("Painting"); 
	       paintNodesItem.setSelected(true); 
	       paintNodesItem.setText("Show Node Location Info"); 
	       paintNodesItem.setActionCommand("Show Node Location Info"); 
	       paintMenu.add(paintNodesItem); 
	       paintEdgesItem.setSelected(true); 
	       */

	    //mainFrame.PacketAnalyzersMenu.add(mainMenu);
	    //this last command adds this entire menu to the main PacketAnalyzers menu 
	    //}} 

	    //{{REGISTER_LISTENERS 
	    packetProcessingCheckBox.addItemListener(this); 
	    backgroundProcessingCheckBox.addItemListener(this); 
	    propertiesItem.addActionListener(this); 
	    saveNodesItem.addActionListener(this); 
	    loadNodesItem.addActionListener(this); 
	    saveEdgesItem.addActionListener(this); 
	    loadEdgesItem.addActionListener(this); 
	    paintNodesItem.addItemListener(this); 
	    paintEdgesItem.addItemListener(this); 
	    paintScreenItem.addItemListener(this); 
	    //}} 
	} 

	//---------------------------------------------------------------------- 
	//EVENT HANDLERS 
	//The following two functions handle menu events 
	//The functions following this are the event handling functions 
	public void actionPerformed(ActionEvent e) 
	{ 
	    Object object = e.getSource(); 
	    /*
	       if (object == saveNodesItem) 
	       SaveNodes(); 
	       else if (object == loadNodesItem) 
	       LoadNodes(); 
	       else if (object == saveEdgesItem) 
	       SaveEdges(); 
	       else if (object == loadEdgesItem) 
	       LoadEdges(); 
	       */
	    if (object == propertiesItem) 
		showOptionsDialog(); 

	} 

	public void itemStateChanged(ItemEvent e) 
	{ 
	    Object object = e.getSource(); 

	    if (object == packetProcessingCheckBox) 
		togglePacketProcessing(); 
	    /*
	       else if (object == backgroundProcessingCheckBox) 
	       toggleBackgroundProcessing(); 
	       else if (object == paintNodesItem) 
	       toggleNodePainting(); 
	       else if (object == paintEdgesItem) 
	       toggleEdgePainting(); 

	       else if (object == paintScreenItem) 
	       toggleScreenPainting(); 
	       */
	}		 
	//EVENT HANDLERS 
	//---------------------------------------------------------------------- 


	//------------------------------------------------------------------------ 
	//****---TOGGLE PACKET PROCESSING 
	//This function will either register or de-register this PacketAnalyzer 
	//as a PacketEventListener.   
	public void togglePacketProcessing() { 
	    if(packetProcessingCheckBox.isSelected()) {
		//Note: the following syntax "RoutingTableAnalyzer.this" allows an inner class to refer to its enclosing class 
		//MainClass.addPacketEventListener(RoutingTableAnalyzer.this);
		//start the background thread of the enclosing packetAnalyzer 
	    } else { 
		//MainClass.removePacketEventListener(RoutingTableAnalyzer.this);
		//stop the background thread of the enclosing packetAnalyzer 
	    }	
	}
    }
}
